<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class Graphing extends \Zeedhi\Framework\Controller\Simple {

    protected $graphingService;
    protected $util;

    public function __construct(\Service\Graphing $graphingService, \Util\Util $util){
        $this->graphingService = $graphingService;
        $this->util = $util;
    }

    public function getProductGroups(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDCLIENTE    = $params[0]['value'];
            $CDCONSUMIDOR = $params[1]['value'];
            $DTINI        = $this->util->formatDate($params[2]['value']);
            $DTFIM        = $this->util->formatDate($params[3]['value']);
            $productGroups = $this->graphingService->getProductGroups($CDCLIENTE, $CDCONSUMIDOR, $DTINI, $DTFIM);

            $mediaConsumo = '0,00';
            $mediaConsumoQuery = $this->graphingService->getMediaConsumo($CDCLIENTE,$CDCONSUMIDOR,$DTINI,$DTFIM);
            if(count($mediaConsumoQuery) > 0){
                if($mediaConsumoQuery[0]['MEDIACONSUMO'] != null){
                    $mediaConsumo = $mediaConsumoQuery[0]['MEDIACONSUMO'];
                }
            }
            // TODO: THIS IS FIXED IN THE CODE.
            $values = array(
                array(
                    'NAME' => ' Data Inicial',
                    'VALUE' => $DTINI.' '
                ),array(
                    'NAME' => ' Data Final',
                    'VALUE' => $DTFIM.' '
                ),
                array(
                    'NAME' => ' Média de Gasto Diária',
                    'VALUE' => $mediaConsumo.' '
                )
            );

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductGroupRepository', $productGroups));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ValuesRepository', $values));
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getSubgroups(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDCLIENTE    = $params[0]['value'];
            $CDCONSUMIDOR = $params[1]['value'];
            $DTINI        = $this->util->formatDate($params[2]['value']);
            $DTFIM        = $this->util->formatDate($params[3]['value']);
            $CDGRUPPROD   = $params[4]['value'];

            if ($CDGRUPPROD == "NULL"){
                $productGroups = $this->graphingService->getProductGroups($CDCLIENTE, $CDCONSUMIDOR, $DTINI, $DTFIM);
                if (!empty($productGroups)){
                    $CDGRUPPROD = $productGroups[0]['CDGRUPPROD'];
                }
            }

            $productSubgroup = $this->graphingService->getSubgroups($CDCLIENTE, $CDCONSUMIDOR, $DTINI, $DTFIM, $CDGRUPPROD);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductGroupRepository', $productSubgroup));
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getProducts(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDCLIENTE    = $params[0]['value'];
            $CDCONSUMIDOR = $params[1]['value'];
            $DTINI        = $this->util->formatDate($params[2]['value']);
            $DTFIM        = $this->util->formatDate($params[3]['value']);
            $CDGRUPPROD   = $params[4]['value'];
            $CDSUBGRPROD  = $params[5]['value'];

            $products = $this->graphingService->getProducts($CDCLIENTE, $CDCONSUMIDOR, $DTINI, $DTFIM, $CDGRUPPROD, $CDSUBGRPROD);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductsRepository', $products));
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    //---------- Nutritional Vision

    public function getProductGroupsNutri(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDCLIENTE    = $params[0]['value'];
            $CDCONSUMIDOR = $params[1]['value'];
            $DTINI        = $this->util->formatDate($params[2]['value']);
            $DTFIM        = $this->util->formatDate($params[3]['value']);
            $productGroups = $this->graphingService->getProductGroupsNutri($CDCLIENTE, $CDCONSUMIDOR, $DTINI, $DTFIM);

            // TODO: THIS IS FIXED IN THE CODE.
            $values = array(
                array(
                    'NAME' => 'Data Inicial',
                    'VALUE' => $DTINI
                ),array(
                    'NAME' => 'Data Final',
                    'VALUE' => $DTFIM
                )
            );

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductGroupNutriRepository', $productGroups));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ValuesNutriRepository', $values));
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getSubgroupsNutri(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDCLIENTE    = $params[0]['value'];
            $CDCONSUMIDOR = $params[1]['value'];
            $DTINI        = $this->util->formatDate($params[2]['value']);
            $DTFIM        = $this->util->formatDate($params[3]['value']);
            $CDGRUPPROD   = $params[4]['value'];

            $productSubgroup = $this->graphingService->getSubgroupsNutri($CDCLIENTE, $CDCONSUMIDOR, $DTINI, $DTFIM, $CDGRUPPROD);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductGroupNutriRepository', $productSubgroup));
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getProductsNutri(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDCLIENTE    = $params[0]['value'];
            $CDCONSUMIDOR = $params[1]['value'];
            $DTINI        = $this->util->formatDate($params[2]['value']);
            $DTFIM        = $this->util->formatDate($params[3]['value']);
            $CDGRUPPROD   = $params[4]['value'];
            $CDSUBGRPROD  = $params[5]['value'];

            $products = $this->graphingService->getProductsNutri($CDCLIENTE, $CDCONSUMIDOR, $DTINI, $DTFIM, $CDGRUPPROD, $CDSUBGRPROD);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductsNutriRepository', $products));
        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }
}