<?php

namespace Service;

class Restriction {

    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager){
        $this->entityManager = $entityManager;
    }

    //********************//
    // VALUE RESTRICTIONS //
    //********************//

    public function getValueRestrictions($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_VALUE_RESTRICTIONS, $params);
    }

    public function getUnrestrictedFamilies($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_UNRESTRICTED_FAMILIES, $params);
    }

    public function addValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $VRCONSDIARAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':VRCONSDIARAL' => $VRCONSDIARAL
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::ADD_VALUE_RESTRICTION, $params);
    }

    public function updateValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $VRCONSDIARAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':VRCONSDIARAL' => $VRCONSDIARAL
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_VALUE_RESTRICTION, $params);
    }

    public function deleteValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD,
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::DELETE_VALUE_RESTRICTION, $params);
    }

    public function getRestrictedProducts($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_RESTRICTED_PRODUCTS, $params);
    }

    //**********************//
    // PRODUCT RESTRICTIONS //
    //**********************//

    public function getProductRestrictions($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            ':CDFILIAL'     => $CDFILIAL,
            ':CDCLIENTE'    => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_RESTRICTIONS, $params);
        foreach($result as &$restrictionItem){
            if ($restrictionItem['QTCONSDIARAL'] === '0'){
                $restrictionItem['QTCONSDIARAL'] = 'não pode consumir';
            }
        }
        return $result;
    }

    public function getConsumerFamilies($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CONSUMER_FAMILIES, $params);
    }

    public function getFamilyProducts($CDFILIAL, $CDFAMILISALD, $CDCLIENTE, $CDCONSUMIDOR){
        $params = array(
            ':CDFILIAL'     => $CDFILIAL,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':CDCLIENTE'    => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_FAMILY_PRODUCTS, $params);
    }

    public function addProductRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $CDPRODUTO, $QTCONSDIARAL, $IDBLOQDIARAL){
        $params = array(
            ':CDFILIAL'     => $CDFILIAL,
            ':CDCLIENTE'    => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':CDPRODUTO'    => $CDPRODUTO,
            ':QTCONSDIARAL' => $QTCONSDIARAL,
            ':IDBLOQDIARAL' => $IDBLOQDIARAL
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::ADD_PRODUCT_RESTRICTION, $params);
    }

    public function updateProductRestriction($QTCONSDIARAL, $CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $CDPRODUTO, $IDBLOQDIARAL){
        $params = array(
            ':QTCONSDIARAL' => $QTCONSDIARAL,
            ':CDFILIAL'     => $CDFILIAL,
            ':CDCLIENTE'    => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':CDPRODUTO'    => $CDPRODUTO,
            ':IDBLOQDIARAL' => $IDBLOQDIARAL
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_PRODUCT_RESTRICTION, $params);
    }

    public function deleteProductRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $CDPRODUTO){
        $params = array(
            ':CDFILIAL'     => $CDFILIAL,
            ':CDCLIENTE'    => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':CDPRODUTO'    => $CDPRODUTO
        );
        return $this->entityManager->getConnection()->executeQuery(\Util\Query::DELETE_PRODUCT_RESTRICTION, $params);
    }

    public function getValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD
        );
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_VALUE_RESTRICTION, $params);
    }

}