using System;
using System.EnterpriseServices;
using System.IO;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;

namespace IMPNF
{
    public interface IMPNFInterface
    {
        int IniciaPorta(int codigoImpressora, string porta);
        int ComandoTX(int codigoImpressora, string comando, int tamanhoComando);
        int FormataTX(int codigoImpressora, string texto, int tipoLetra, int italico, int sublinhado, int expandido, int enfatizado, int porta);
        int FechaPorta(int codigoImpressora, int porta);
        int ConfiguraCodigoBarras(int codigoImpressora, int altura, int largura, int posicao, int fonte, int margem);
        int ImprimeCodigoBarrasCODE128(int codigoImpressora, string texto, int porta);
        int ImprimeCodigoQRCODE(int codigoImpressora, int errorCorrectionLevel, int moduleSize, int codeType, int QRCodeVersion, int encodingModes, string codeQr, int porta);
        int Le_Status(int codigoImpressora);
        int AlimentaLinhasKiosk(int porta, int tipoPorta, int qtdLinhas);
        int CortaPapel(int codigoImpressora, int porta);
    }

    public class IMPNFClass : ServicedComponent, IMPNFInterface
    {
        public int IniciaPorta(int codigoImpressora, string porta)
        {
            switch (codigoImpressora)
            {
                case 1:
                case 3:
                case 4:
                case 7:
                case 13:
                    return BEMANFDLL.IniciaPorta(porta);
                case 2:
                case 9:
                case 11:
                case 14:
                case 16:
                    if (porta == "USB")
                    {
                        EPSONNFDLL.ConfiguraTaxaSerial(9600);
                    }
                    else
                    {
                        EPSONNFDLL.ConfiguraTaxaSerial(115200);
                    }
                    return EPSONNFDLL.IniciaPorta(porta);
                case 18:
                    int port;
                    IntPtr printerInstance = new IntPtr();
                    printerInstance = IntPtr.Zero;
                    ELGINi9.PrtPrinterCreator(ref printerInstance, "TP806");
                    port = ELGINi9.PrtPortOpen(printerInstance, "USB");
                    ELGINi9.PrtPrinterDestroy(printerInstance);

                    // On i9 printer, 0 means succes so I'm returning 1 if 0 and -1 if something wrong happens
                    return port == 0 ? 1 : -1;
                case 20:
                    return ELGINKIOSK.KIOSK_OpenUsb();
                default:
                    return -1;
            }
        }


        public int ComandoTX(int codigoImpressora, string comando, int tamanhoComando)
        {
            switch (codigoImpressora)
            {
                case 1:
                case 3:
                case 4:
                case 7:
                case 13:
                    return BEMANFDLL.ComandoTX(comando, tamanhoComando);
                case 2:
                case 9:
                case 11:
                case 14:
                case 16:
                    return EPSONNFDLL.ComandoTX(comando, tamanhoComando);
                case 18:
                    return 1;
                case 20:
                    return 1;
                default:
                    return 0;
            }
        }

        public int FormataTX(int codigoImpressora, string texto, int tipoLetra, int italico, int sublinhado, int expandido, int enfatizado, int porta = 0)
        {
            switch (codigoImpressora)
            {
                case 1:
                case 3:
                case 4:
                case 7:
                case 13:
                    return BEMANFDLL.FormataTX(texto, tipoLetra, italico, sublinhado, expandido, enfatizado);
                case 2:
                case 9:
                case 11:
                case 14:
                case 16:
                    return EPSONNFDLL.FormataTX(texto, tipoLetra, italico, sublinhado, expandido, enfatizado);
                case 18:
                    IntPtr printerInstance = new IntPtr();
                    printerInstance = IntPtr.Zero;
                    ELGINi9.PrtPrinterCreator(ref printerInstance, "TP806");
                    ELGINi9.PrtPortOpen(printerInstance, "USB");
                    ELGINi9.PrtPrinterInitialize(printerInstance);
                    ELGINi9.PrtSetTextFont(printerInstance, 1);

                    byte[] data = Encoding.GetEncoding("GB2312").GetBytes(texto);

                    ELGINi9.PrtPrintText(printerInstance, data, 1, 0, 0);
                    ELGINi9.PrtPrinterDestroy(printerInstance);
                    return 1;
                case 20:
                    return ELGINKIOSK.KIOSK_S_Textout(porta, 2, texto, 0, 1, 1, 1, 0);
                default:
                    return 0;
            }
        }

        public int FechaPorta(int codigoImpressora, int porta = 0)
        {
            switch (codigoImpressora)
            {
                case 1:
                case 3:
                case 4:
                case 7:
                case 13:
                    return BEMANFDLL.FechaPorta();
                case 2:
                case 9:
                case 11:
                case 14:
                case 16:
                    return EPSONNFDLL.FechaPorta();
                case 18:
                    int port;
                    IntPtr printerInstance = new IntPtr();
                    printerInstance = IntPtr.Zero;
                    ELGINi9.PrtPrinterCreator(ref printerInstance, "TP806");
                    port = ELGINi9.PrtPortClose(printerInstance);
                    ELGINi9.PrtPrinterDestroy(printerInstance);
                    return port;
                case 20:
                    if (porta != 0)
                    {
                        return ELGINKIOSK.KIOSK_CloseUsb(porta);
                    }
                    else
                    {
                        return -1;
                    }
                default:
                    return 0;
            }
        }

        public int ConfiguraCodigoBarras(int codigoImpressora, int altura, int largura, int posicao, int fonte, int margem)
        {
            switch (codigoImpressora)
            {
                case 1:
                case 3:
                case 4:
                case 7:
                case 13:
                    return BEMANFDLL.ConfiguraCodigoBarras(altura, largura, posicao, fonte, margem);
                case 2:
                case 9:
                case 11:
                case 14:
                case 16:
                    return EPSONNFDLL.ConfiguraCodigoBarras(altura, largura, posicao, fonte, margem);
                case 18:
                    return 1;
                case 20:
                    return 1;
                default:
                    return 0;
            }
        }

        public int ImprimeCodigoBarrasCODE128(int codigoImpressora, string texto, int porta = 0)
        {
            switch (codigoImpressora)
            {
                case 1:
                case 3:
                case 4:
                case 7:
                case 13:
                    return BEMANFDLL.ImprimeCodigoBarrasCODE128(texto);
                case 2:
                case 9:
                case 11:
                case 14:
                case 16:
                    return EPSONNFDLL.ImprimeCodigoBarrasCODE128(texto);
                case 18:
                    return 1;
                case 20:
                     ELGINKIOSK.KIOSK_S_SetAlignMode(porta, 2, 1);         
                     String textConvertido = "{B" + texto;
                     ELGINKIOSK.KIOSK_S_PrintBarcode(porta, 2, "{B" + texto, 5, 73, 2, 50, 1, 0, textConvertido.Length);
                     ELGINKIOSK.KIOSK_S_SetAlignMode(porta, 2, 0);
                     return 1;
                default:
                    return 0;
            }
        }


        public int ImprimeCodigoQRCODE(int codigoImpressora, int errorCorrectionLevel, int moduleSize, int codeType, int QRCodeVersion, int encodingModes, string codeQr, int porta = 0)
        {
            unsafe
            {
                switch (codigoImpressora)
                {
                    case 1:
                    case 3:
                    case 4:
                    case 7:
                    case 13:
                        return BEMANFDLL.ImprimeCodigoQRCODE(errorCorrectionLevel, moduleSize, codeType, QRCodeVersion, encodingModes, codeQr);
                    case 2:
                    case 9:
                    case 11:
                    case 14:
                    case 16:
                        return EPSONNFDLL.ImprimeCodigoQRCODE(errorCorrectionLevel, moduleSize, codeType, QRCodeVersion, encodingModes, codeQr);
                    case 18:
                        int printResult;
                        IntPtr printerInstance = new IntPtr();
                        printerInstance = IntPtr.Zero;
                        ELGINi9.PrtPrinterCreator(ref printerInstance, "TP806");
                        ELGINi9.PrtPortOpen(printerInstance, "USB");
                        ELGINi9.PrtPrinterInitialize(printerInstance);

                        string urlMigreMe = string.Format(codeQr);

                        string response;
                        try
                        {
                            var client = new WebClient();
                            response = client.DownloadString(urlMigreMe);
                        }
                        catch (Exception e)
                        {
                            response = "QR Code invlido";
                        }  
                        
                        printResult = ELGINi9.PrtPrintBarCode(printerInstance, 104, response, 3, 60, 1, 3);
                        ELGINi9.PrtPrinterDestroy(printerInstance);

                        // On i9 printer, 0 means succes so I'm returning 1 if 0 and -1 if something wrong happens
                        return printResult == 0 ? 1 : -1;
                    case 20:
                        ELGINKIOSK.KIOSK_S_SetAlignMode(porta, 2, 1);
                        byte[] conf = new byte[] { 0X1D, 0X6F, 0X00, 4, 0X00, 0X02, 0X0A };
                        int confLength = conf.Length - 1;
                        ELGINKIOSK.KIOSK_WriteData(porta, 2, conf, confLength);

                        int tamQrCode = codeQr.Length + 8;
                        conf = new byte[tamQrCode];
                        conf[0] = 0X1D;
                        conf[1] = 0X6B;
                        conf[2] = 0X0B;
                        conf[3] = 0X4C;
                        conf[4] = 0X41;
                        conf[5] = 0X2C;

                        for (int i = 0; i < codeQr.Length; i++)
                        {
                            char letter = codeQr[i];
                             byte value = (byte) letter;
                            conf[i+6] = value;
                        }

                        confLength = conf.Length - 1;
                        conf[confLength - 1] = 0X00;
                        conf[confLength] = 0X0A;
                        ELGINKIOSK.KIOSK_WriteData(porta, 2, conf, confLength);

                        ELGINKIOSK.KIOSK_S_SetAlignMode(porta, 2, 0);
                        return 1;
                    default:
                        return 0;
                }
            }
        }

        public int Le_Status(int codigoImpressora)
        {
            switch (codigoImpressora)
            {
                case 1:
                case 3:
                case 4:
                case 7:
                case 13:
                    return BEMANFDLL.Le_Status();
                case 2:
                case 9:
                case 11:
                case 14:
                case 16:
                    return EPSONNFDLL.Le_Status();
                case 18:
                    return 1;
                case 20:
                    return 1;
                default:
                    return 0;
            }
        }

        public int AlimentaLinhasKiosk(int porta, int tipoPorta, int qtdLinhas)
        {
            return ELGINKIOSK.KIOSK_FeedLines(porta, tipoPorta, qtdLinhas);
        }

        public int CortaPapel(int codigoImpressora, int porta = 0)
        {
            switch (codigoImpressora) {
                case 20:
                    return ELGINKIOSK.KIOSK_CutPaper(porta, 2, 1, 10);
                default:
                    return -1;
            }
            
        }
    }
}
