<?php

namespace Odhen\API\Service;

class Comanda {
	
	protected $entityManager;
	protected $util;
	protected $consumidor;
	protected $preco;

	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
		\Odhen\API\Util\Util $util,
		\Odhen\API\Service\Consumidor $consumidor,
		\Odhen\API\Service\Preco $preco)  {
			
		$this->entityManager = $entityManager;
		$this->util          = $util;
		$this->consumidor    = $consumidor;
		$this->preco    	 = $preco;
	}

	const MOVIMENTACAO_ENTRADA = 'E';
    const MOVIMENTACAO_SAIDA   = 'S';

	const status = array(
        '1' => 'Em Aberto',
        '2' => 'Conta Solicitada',
        '3' => 'Pago',
        '4' => 'Cancelado',
        '5' => 'Transferido/Despachado',
        '6' => 'Não Pago',
        'P' => 'Impresso',
        'O' => 'Concluido'
    );

    const TIPO_PAGAMENTO_DINHEIRO = '001';

	/**
	 * @param  [array] produtos array com as informações dos produtos
	 * @param  [array] pagamento array com os dados do pagamento
	 * @param  [array] dadosConsumidor array com os dados do consumidor
	 * @param  [array] dadosFilial array com os dados da filial
	 * @param  [array] dadosPedido array com os dados do pedido como, isTogo, NRMESA NRPESMESA
	 */
	public function comanda($produtos, $pagamento, $dadosConsumidor, $dadosFilial, $dadosPedido){		
		try {
			
			$dadosConsumidor = $this->consumidor->populaDadosConsumidor($dadosConsumidor);
			$enderecoComanda = self::defineEnderecoComanda($dadosConsumidor, $dadosFilial);
			if($enderecoComanda['error'] == false){
				$this->defineRetiradaBalcao($dadosPedido);
				$validaDados = self::validaDadosComanda($dadosFilial);
			
				if($validaDados['error'] == false){
					$dadosNovaComanda = self::abreComanda($validaDados['data'], $dadosFilial, $dadosConsumidor, $dadosPedido, $enderecoComanda['enderecoComanda'], $pagamento);

					if($dadosNovaComanda['error'] == false){
						$dadosPagamento = self::registraPagamento($pagamento['tipoRecebimento'], $dadosConsumidor, $dadosFilial, $dadosNovaComanda['dadosComanda']);
						/*TODO: Pagamento online*/
						if($dadosPagamento['error'] == false){
							$dadosProdutos = self::preparaProdutos($produtos, $dadosFilial, $dadosConsumidor, $dadosNovaComanda['dadosComanda'], $pagamento);

							if ($dadosProdutos['error'] == false) {
								$retornoInsereProdutos = self::insereProdutos($dadosFilial, $dadosNovaComanda['dadosComanda'], $dadosProdutos['produtosTratados'], $validaDados['data']);

								if ($retornoInsereProdutos['error'] == false) {
									return array(
										'error' => false,
										'dadosComanda' => $dadosNovaComanda['dadosComanda'],
										'dadosPagamento' => $dadosPagamento['dadosPagamento']);	
								} else {
									return $retornoInsereProdutos;
								}

							} else {
								return $dadosProdutos;
							}

						} else {
							return $dadosPagamento;
						}
					} else {
						return $dadosNovaComanda;
					}
				} else {
					return $validaDados;
				}
			} else {
				return $enderecoComanda;
			} 
		} catch(\Exception $e){
			$result = array(
				'error' => true,
				'message' => $e->getMessage()
			);
			return $result;
		}
		
	}

	public function trataDivisaoDesconto($assembledProducts, $orderDiscount, $pagamento){
		$produtosCupom = array();
		if(isset($pagamento['promoCoupon'])){
			$produtosCupom = $pagamento['promoCoupon']['produtos'];
		}
        if(!empty($produtosCupom)){
            foreach($produtosCupom as $key => $value){
                foreach($assembledProducts as &$product){
                    if($product['CDPRODUTO'] == str_replace('.','',$key)){
                        $product['DESC'] = $product['DESC'] + $value['desconto'];
                    }
                }
            }
            
            return $assembledProducts;
            
        }else{
            $orderValue = 0;
			
			foreach($assembledProducts as $product){
				$itemDiscount = isset($product['DESC']) ? $product['DESC'] : 0;
				$orderValue  += ($product['VRPRECITEM'] - $itemDiscount) * $product['QUANTIDADE'];
			}
			
			foreach($assembledProducts as &$product){
				$itemDiscount    = isset($product['DESC']) ? $product['DESC']  : 0;
				$discountToShare = number_format(((($product['VRPRECITEM'] - $itemDiscount) * $product['QUANTIDADE']) * $orderDiscount) / $orderValue,2);
				$product['DESC'] = $product['DESC'] + $discountToShare;
			}
			
			return $assembledProducts;
        }
    }

	public function defineEnderecoComanda($dadosConsumidor, $dadosFilial){
		try {
			$params = array('CDFILIAL' => $dadosFilial['CDFILIAL']);
			$enderecoFilial = $this->entityManager->getConnection()->fetchAssoc(self::ENDERECO_LOJA, $params);
			if(empty($dadosConsumidor['NRCEPCONS'])){
				$cepComanda = $enderecoFilial['NRCEPFILI'];
				$dsEndeComanda = $enderecoFilial['DSENDEFILI'];
				$dsBairroComanda = $enderecoFilial['NMBAIRFILI'];
			} else {
				$cepComanda = $dadosConsumidor['NRCEPCONS'];
				$dsEndeComanda = $dadosConsumidor['DSENDECONS'];
				$dsBairroComanda = $dadosConsumidor['NMBAIRRO'];
			}
			$enderecoComanda = array(
				'NRCEPCONSCOMAND' => $cepComanda,
				'DSBAIRRO' => $dsBairroComanda,
				'DSENDECONSCOMAN' => $dsEndeComanda);
			return array(
				'error' => false,
				'enderecoComanda' => $enderecoComanda);
		} catch (\Exception $e) {
			return array(
        		'error' => true,
        		'message' => $e->getMessage()
        	);
		}
	}

	const ENDERECO_LOJA = "
		SELECT FL.NMFILIAL, MP.NMMUNICIPIO, EF.DSENDEFILI, EF.NMBAIRFILI,
	   		EF.SGESTADO, EF.NRCEPFILI, EF.NRTELEFILI
  		FROM ENDEFILI EF
  				JOIN MUNICIPIO MP ON MP.CDMUNICIPIO = EF.CDMUNICIPIO
				JOIN FILIAL FL ON FL.CDFILIAL = EF.CDFILIAL
		WHERE EF.CDFILIAL = :CDFILIAL
				AND EF.IDTPENDEFILI = 'P'
   	"; 

	public function insereProdutos($dadosFilial, $dadosComanda, $dadosProdutos, $dadosGerais){
        try {
        	$connection = $this->entityManager->getConnection();
        	$CDCONTADOR = 'ITCOMANDAVEN' . $dadosFilial['CDFILIAL'];
        	$NRORG = 1;
        	$nrSeqProdCup = $this->util->geraCodigo($connection, $CDCONTADOR, $NRORG, 1, 10);
            foreach($dadosProdutos as $produto){
            	if(!isset($produto['IDIMPPRODUTO']) || $produto['IDIMPPRODUTO'] != '1'){
            		$idStPrComVen = $this->getTheThing($dadosFilial['CDFILIAL'], $dadosFilial['CDLOJA'], $produto['CDPRODUTO'], $dadosGerais['IDCONTROPROD']);
	                $CDCONTADOR = 'ITCOMANDAVEN'.$dadosFilial['CDFILIAL'].$dadosComanda['NRCOMANDA'];
	            	$nrProdComVen = $this->util->geraCodigo($connection, $CDCONTADOR, $NRORG, 1, 6);

		            if(isset($produto['comboCart'])){
		            	foreach($produto['comboCart'] as $combo){
		            		foreach ($combo['products'] as $promoItem) {
		            			$insertParams = array(
				                    ':CDFILIAL' => $dadosFilial['CDFILIAL'],
				                    ':NRVENDAREST' => $dadosComanda['NRVENDAREST'],
				                    ':NRCOMANDA' => $dadosComanda['NRCOMANDA'],
				                    ':NRPRODCOMVEN' => $nrProdComVen,
				                    ':CDPRODUTO' => $promoItem['CDPRODUTO'],
				                    ':QTPRODCOMVEN' => $promoItem['QUANTIDADE'],
				                    ':VRPRECCOMVEN' => $promoItem['VRPRECITEM'],
				                    ':TXPRODCOMVEN' => $promoItem['TXPRODCOMVEN'],
				                    ':IDSTPRCOMVEN' => $idStPrComVen,
				                    ':VRDESCCOMVEN' => isset($promoItem['DESC']) ? $promoItem['DESC'] : 0,
				                    ':NRLUGARMESA' => '1',
				                    ':IDPRODIMPFIS' => 'N',
				                    ':CDLOJA' => $dadosFilial['CDLOJA'],
				                    ':VRACRCOMVEN' => isset($promoItem['ACRE']) ? $promoItem['ACRE'] : 0,
				                    ':NRSEQPRODCOM' => isset($promoItem['NRSEQPRODCOM']) ? $promoItem['NRSEQPRODCOM'] : null,
				                    ':NRSEQPRODCUP' => $nrSeqProdCup,
				                    ':VRPRECCLCOMVEN' => 0,
				                    ':CDCAIXACOLETOR' => $dadosFilial['CDCAIXA'],
				                    ':CDPRODPROMOCAO' => isset($promoItem['CDPRODPROMOCAO']) ? $promoItem['CDPRODPROMOCAO'] : null,
				                    ':CDVENDEDOR' => $dadosGerais['CDVENDEDOR']
				                );
				                $this->entityManager->getConnection()->executeQuery(self::INSERT_PRODUCT, $insertParams);
							}
						}
		            } else {
		                $insertParams = array(
		                    ':CDFILIAL' => $dadosFilial['CDFILIAL'],
		                    ':NRVENDAREST' => $dadosComanda['NRVENDAREST'],
		                    ':NRCOMANDA' => $dadosComanda['NRCOMANDA'],
		                    ':NRPRODCOMVEN' => $nrProdComVen,
		                    ':CDPRODUTO' => $produto['CDPRODUTO'],
		                    ':QTPRODCOMVEN' => $produto['QUANTIDADE'],
		                    ':VRPRECCOMVEN' => $produto['VRPRECITEM'],
		                    ':TXPRODCOMVEN' => $produto['TXPRODCOMVEN'],
		                    ':IDSTPRCOMVEN' => $idStPrComVen,
		                    ':VRDESCCOMVEN' => isset($produto['DESC']) ? $produto['DESC'] : 0,
		                    ':NRLUGARMESA' => '1',
		                    ':IDPRODIMPFIS' => 'N',
		                    ':CDLOJA' => $dadosFilial['CDLOJA'],
		                    ':VRACRCOMVEN' => isset($produto['ACRE']) ? $produto['ACRE'] : 0,
		                    ':NRSEQPRODCOM' => isset($produto['NRSEQPRODCOM']) ? $produto['NRSEQPRODCOM'] : null,
		                    ':NRSEQPRODCUP' => $nrSeqProdCup,
		                    ':VRPRECCLCOMVEN' => 0,
		                    ':CDCAIXACOLETOR' => $dadosFilial['CDCAIXA'],
		                    ':CDPRODPROMOCAO' => isset($produto['CDPRODPROMOCAO']) ? $produto['CDPRODPROMOCAO'] : null,
		                    ':CDVENDEDOR' => $dadosGerais['CDVENDEDOR']
		                );
		                $this->entityManager->getConnection()->executeQuery(self::INSERT_PRODUCT, $insertParams);
		            }
            	} else {
	                $idStPrComVen = $this->getTheThing($dadosFilial['CDFILIAL'], $dadosFilial['CDLOJA'], $produto['CDPRODUTO'], $dadosGerais['IDCONTROPROD']);
	                $CDCONTADOR = 'ITCOMANDAVEN'.$dadosFilial['CDFILIAL'].$dadosComanda['NRCOMANDA'];
	            	$nrProdComVen = $this->util->geraCodigo($connection, $CDCONTADOR, $NRORG, 1, 6);
	                $insertParams = array(
	                    ':CDFILIAL' => $dadosFilial['CDFILIAL'],
	                    ':NRVENDAREST' => $dadosComanda['NRVENDAREST'],
	                    ':NRCOMANDA' => $dadosComanda['NRCOMANDA'],
	                    ':NRPRODCOMVEN' => $nrProdComVen,
	                    ':CDPRODUTO' => $produto['CDPRODUTO'],
	                    ':QTPRODCOMVEN' => $produto['QUANTIDADE'],
	                    ':VRPRECCOMVEN' => $produto['VRPRECITEM'],
	                    ':TXPRODCOMVEN' => $produto['TXPRODCOMVEN'],
	                    ':IDSTPRCOMVEN' => $idStPrComVen,
	                    ':VRDESCCOMVEN' => isset($produto['DESC']) ? $produto['DESC'] : 0,
	                    ':NRLUGARMESA' => '1',
	                    ':IDPRODIMPFIS' => 'N',
	                    ':CDLOJA' => $dadosFilial['CDLOJA'],
	                    ':VRACRCOMVEN' => isset($produto['ACRE']) ? $produto['ACRE'] : 0,
	                    ':NRSEQPRODCOM' => isset($produto['NRSEQPRODCOM']) ? $produto['NRSEQPRODCOM'] : null,
	                    ':NRSEQPRODCUP' => $nrSeqProdCup,
	                    ':VRPRECCLCOMVEN' => 0,
	                    ':CDCAIXACOLETOR' => $dadosFilial['CDCAIXA'],
	                    ':CDPRODPROMOCAO' => isset($produto['CDPRODPROMOCAO']) ? $produto['CDPRODPROMOCAO'] : null,
	                    ':CDVENDEDOR' => $dadosGerais['CDVENDEDOR']
	                );
	                $this->entityManager->getConnection()->executeQuery(self::INSERT_PRODUCT, $insertParams);

		            // SmartPromo products that aren't printed (ITCOMANDAEST).
		            if(isset($produto['comboCart'])){
		            	foreach($produto['comboCart'] as $combo){
		            		foreach ($combo['products'] as $promoItem) {
			            		$promoParams = array(
				                    ':CDFILIAL' => $dadosFilial['CDFILIAL'],
				                    ':NRVENDAREST' => $dadosComanda['NRVENDAREST'],
				                    ':NRCOMANDA' => $dadosComanda['NRCOMANDA'],
				                    ':NRPRODCOMVEN' => $nrProdComVen,
				                    ':CDPRODUTO' => $promoItem['CDPRODUTO'],
				                    ':QTPROCOMEST' => $produto['QUANTIDADE'],
				                    ':VRPRECCOMEST' => $promoItem['VRPRECITEM'],
				                    ':VRDESITCOMEST' => $promoItem['VRDESPRODPROMOC'],
				                    ':TXPRODCOMVENEST' => $promoItem['TXPRODCOMVEN']
				                );
				                $this->entityManager->getConnection()->executeQuery(self::INSERT_SMARTPROMO, $promoParams);
							}
						}
		            }
            	}
            }
            return array(
            	'error' => false,
            	'message' => 'Produtos Inseridos');
        } catch (\Exception $e) {
            return array(
        		'error' => true,
        		'message' => $e->getMessage()
        	);
        }
    }

    const INSERT_PRODUCT = "
		INSERT INTO ITCOMANDAVEN
			(CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN,
			CDPRODUTO, QTPRODCOMVEN, VRPRECCOMVEN, TXPRODCOMVEN,
			IDSTPRCOMVEN, VRDESCCOMVEN, NRLUGARMESA, DTHRINCOMVEN,
			IDPRODIMPFIS, CDLOJA, VRACRCOMVEN, NRSEQPRODCOM, NRSEQPRODCUP,
			VRPRECCLCOMVEN, CDCAIXACOLETOR, CDPRODPROMOCAO, CDVENDEDOR)
		VALUES
			(:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN,
			:CDPRODUTO, :QTPRODCOMVEN, :VRPRECCOMVEN, :TXPRODCOMVEN,
			:IDSTPRCOMVEN, :VRDESCCOMVEN, :NRLUGARMESA, SYSDATE,
			:IDPRODIMPFIS, :CDLOJA, :VRACRCOMVEN, :NRSEQPRODCOM, :NRSEQPRODCUP,
			:VRPRECCLCOMVEN, :CDCAIXACOLETOR, :CDPRODPROMOCAO, :CDVENDEDOR)
	";

	const INSERT_SMARTPROMO = "
		INSERT INTO ITCOMANDAEST(
			CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN,
			CDPRODUTO, QTPROCOMEST, VRPRECCOMEST, VRDESITCOMEST,
			TXPRODCOMVENEST
		)
		VALUES(
			:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN,
			:CDPRODUTO, :QTPROCOMEST, :VRPRECCOMEST, :VRDESITCOMEST,
			:TXPRODCOMVENEST
		)
	";

	public function getTheThing($CDFILIAL, $CDLOJA, $CDPRODUTO, $IDCONTROPROD){
        try {
        	$params = array(
				':CDFILIAL' => $CDFILIAL,
				':CDLOJA' => $CDLOJA,
				':CDPRODUTO' => $CDPRODUTO
			);
        	$theThing = $this->entityManager->getConnection()->fetchAssoc(self::GET_SECTOR_CODE, $params);
            if ($IDCONTROPROD === 'S' && $theThing !== null){
                $finalThing = '1';
            }
            else if ($IDCONTROPROD === 'N'){
                $finalThing = '4';
            }
            else {
                $finalThing = $IDCONTROPROD;
            }
            return $finalThing;
        } catch (\Exception $e) {
        	return array(
        		'error' => true,
        		'message' => $e->getMessage());
        }
    }

    const GET_SECTOR_CODE = "
		SELECT CDSETOR
		  FROM PRODLOJA
		 WHERE CDFILIAL  = :CDFILIAL
		   AND CDLOJA    = :CDLOJA
		   AND CDPRODUTO = :CDPRODUTO
	";

	private function preparaProdutos($produtos, $dadosFilial, $dadosConsumidor, $dadosComanda, $pagamento){
		try {
			$produtosTratados = array();

			foreach ($produtos as &$produto){
				if($produto['IDTIPOCOMPPROD'] != 'C'){
					$detalhesProduto = $this->buscaDetalhesProduto($produto['CDPRODUTO']);
					$preco = $this->preco->buscaPreco($dadosFilial['CDFILIAL'], $dadosConsumidor['CDCLIENTE'], $produto['CDPRODUTO'], $dadosFilial['CDLOJA'], $dadosConsumidor['CDCONSUMIDOR']);
					$detalhesProduto = array_merge($detalhesProduto, $preco);

					if (isset($produto['comboCart'])){
						$NRSEQPRODCOM = $this->buscaPromoProdCode($dadosFilial['CDFILIAL'], $dadosComanda['NRVENDAREST'], $dadosComanda['NRCOMANDA']);
	            		$NRSEQPRODCOM = intval($NRSEQPRODCOM['NRSEQPRODCOM']);
	                	// CODE SEGMENT: NRSEQPRODCOM - same number for all items in a SmartPromo.
		        		$nrSeqProdCom = str_pad((string) ($NRSEQPRODCOM + 1), 3, '0', STR_PAD_LEFT);
		            	$NRSEQPRODCOM++;
		            	$produto['NRSEQPRODCOM'] = $nrSeqProdCom;
		            	$produto['CDPRODPROMOCAO'] = $produto['CDPRODUTO'];

		            	foreach($produto['comboCart'] as $combo){
		            		foreach ($combo['products'] as &$promoItem) {
			            		$tabelaPreco = $this->preco->buscaTabelaVigente($dadosFilial['CDFILIAL'], $dadosFilial['CDLOJA'], $dadosConsumidor['CDCLIENTE']);
			            		/* Gets SmartPromo product prices. */
			            		$promoPrice = $this->preco->getSmartPromoPrice($produto['CDPRODUTO'], $promoItem['parentComboItem'], $promoItem['CDPRODUTO'], $dadosFilial['CDFILIAL'], $tabelaPreco['CDTABEPREC'], $tabelaPreco['DTINIVGPREC']);
			            		//$promoItem['PRECO'] = $promoItem['price']; check if needed
			            		$promoItem['ACRE'] = 0;
			            		$promoItem = array_merge($promoItem, $promoPrice);
			            		/* Applies discount. */
			            		$promoItem = self::checkDesconto($promoItem); // Discount is apparently not needed - price and discount are inserted as is.
			            		/* Additional values for SmartPromo products. */
			            		$promoItem['NRSEQPRODCOM'] = $nrSeqProdCom;
			            		$promoItem['CDPRODPROMOCAO'] = $produto['CDPRODUTO'];
		            		}
			            }
			        } else {
			        	/* These values are useless for normal products. */
			        	$produto['NRSEQPRODCOM'] = null;
			        	$produto['CDPRODPROMOCAO'] = null;
			        }
			        
		            $detalhesProduto = array_merge($detalhesProduto, $produto);

		            /* Special case to add SmartPromo products that don't depend on the parent product. */
		            if ($detalhesProduto['IDIMPPRODUTO'] == '2'){
		            	foreach($produto['comboCart'] as $combo){
		            		foreach ($combo['products'] as $promoItem) {
		            				array_push($produtosTratados, $promoItem);
							}
			            }
		            } else{
		                array_push($produtosTratados, $detalhesProduto);
		            }
				} else {
					$NRSEQPRODCOM = $this->buscaPromoProdCode($dadosFilial['CDFILIAL'], $dadosComanda['NRVENDAREST'], $dadosComanda['NRCOMANDA']);
            		$NRSEQPRODCOM = intval($NRSEQPRODCOM['NRSEQPRODCOM']);
                	// CODE SEGMENT: NRSEQPRODCOM - same number for all items in a SmartPromo.
	        		$nrSeqProdCom = str_pad((string) ($NRSEQPRODCOM + 1), 3, '0', STR_PAD_LEFT);

					$this->_trataProdutosCombo($produto, $nrSeqProdCom, $produtosTratados, $dadosFilial, $dadosConsumidor);
				}
			}

			if(isset($pagamento['discount'])){
				$produtosTratados = $this->trataDivisaoDesconto($produtosTratados, $pagamento['discount'], $pagamento);
			}

			return array(
				'error' => false,
				'produtosTratados' => $produtosTratados); 
		}catch (\Exception $e) {
			return array(
				'error' => true,
				'message' => 'Erro ao registrar os produtos do pedido. ' . $e->getMessage());
		}
	}

	private function _trataProdutosCombo(&$produto, $nrSeqProdCom, &$produtosTratados, $dadosFilial, $dadosConsumidor){
		$IDTIPCOBRA     = $produto['IDTIPCOBRA'];
		$maiorPreco     = 0;
		$somaDasQuantidades = 0;

		if($IDTIPCOBRA == 'C'){ //IDTIPCOBRA fala se o preço que será cobrado é à partir do produto mais caro ou não.
			foreach($produto['comboCart'] as &$combo){
        		foreach ($combo['products'] as &$comboProduct) {
        			$preco = $this->preco->buscaPreco($dadosFilial['CDFILIAL'], $dadosConsumidor['CDCLIENTE'], $comboProduct['CDPRODUTO'], $dadosFilial['CDLOJA'], $dadosConsumidor['CDCONSUMIDOR']);
        			if(!$preco['error']){
        				$comboProduct['VRPRECITEM'] = $preco['PRECO'];
        			} else {
        				// $comboProduct['VRPRECITEM'] = $comboProduct['VRPRECITEM'] * $somaDasQuantidades;
        			}

        			if($comboProduct['VRPRECITEM'] > $maiorPreco){
        				$maiorPreco = $comboProduct['VRPRECITEM'];
        			}

    				$somaDasQuantidades += $comboProduct['QUANTIDADE'];
				}
            }
		}

		//Calcula a divisão da quantidade nos itens do produto combinado.
		$quantidadeEspecial = 0;
		$quantidade = round(1 / $somaDasQuantidades, 2);
        if ($quantidade * $somaDasQuantidades != 1){
            $quantidadeEspecial = $quantidade + (1 - $quantidade * $somaDasQuantidades);
        }
        //----

        foreach($produto['comboCart'] as $comboCart){
    		foreach ($comboCart['products'] as $comboCartProduct) {
    			if($IDTIPCOBRA == 'C'){
    				$comboCartProduct['VRPRECITEM'] = $maiorPreco;
				}

				if($quantidadeEspecial > 0){
					if($comboCartProduct['QUANTIDADE'] == 1){
						$comboCartProduct['QUANTIDADE'] = $quantidadeEspecial;
					} else if($comboCartProduct['QUANTIDADE'] == $somaDasQuantidades){
						$comboCartProduct['QUANTIDADE'] = 1;
					} else {
						$comboCartProduct['QUANTIDADE'] = ($quantidade * $comboCartProduct['QUANTIDADE']) + ($quantidadeEspecial - $quantidade);
					}

					$quantidadeEspecial = 0;
				} else {
					$comboCartProduct['QUANTIDADE'] = $quantidade * $comboCartProduct['QUANTIDADE'];
				}

				$comboCartProduct['NRSEQPRODCOM'] = $nrSeqProdCom;
				array_push($produtosTratados, $comboCartProduct);
			}
        }
	}

	private function processObservations($product){
        if (!empty($product['selectItems'])){
            $TXPRODCOMVEN = '';
            foreach ($product['selectItems'] as $observation){
                if (isset($observation['DSOCORR'])) {
                    $TXPRODCOMVEN .= $observation['DSOCORR'] . '; ';
                }
            }
            return $TXPRODCOMVEN;
        }
        else {
            return null;
        }
    }

	private function checkDesconto($produto){
        try {
            if ($produto['IDPERVALORDES'] == 'P'){
	            $produto['DESC'] = $produto['VRPRECITEM'] * ($produto['VRDESPRODPROMOC']/100);
            }
            else if ($produto['IDPERVALORDES'] == 'V'){
                $produto['DESC'] = $produto['VRDESPRODPROMOC'];
            }
            else { // Products that haven't had a discount registered.
            	$produto['DESC'] = 0;
            }
            if ($produto['DESC'] > $produto['VRPRECITEM']){
            	$produto['DESC'] = $produto['VRPRECITEM'] - 0.01;
            }
            /* Last digit can't be 9. */
            $produto['DESC'] = intval($produto['DESC']*100)/100;
            return $produto;
        } catch (\Exception $e) {
            return array(
            	'error' => true,
            	'message' => $e->getMessage());
        }
    }

	public function buscaPromoProdCode($CDFILIAL, $NRVENDAREST, $NRCOMANDA){
		$params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':NRVENDAREST' => $NRVENDAREST,
            ':NRCOMANDA' => $NRCOMANDA
        );
		$result = $this->entityManager->getConnection()->fetchAssoc(self::GET_NRSEQPRODCOM, $params);
		return $result;
	}

	const GET_NRSEQPRODCOM = "
		SELECT NVL(MAX(NRSEQPRODCOM),0) AS NRSEQPRODCOM
		  FROM ITCOMANDAVEN
		 WHERE CDFILIAL    = :CDFILIAL
		   AND NRVENDAREST = :NRVENDAREST
		   AND NRCOMANDA   = :NRCOMANDA
		   AND NRSEQPRODCOM IS NOT NULL
	";

	private function buscaDetalhesProduto($CDPRODUTO){
		try {
			$params = array('CDPRODUTO' => $CDPRODUTO);
			$detalhesProduto = $this->entityManager->getConnection()->fetchAssoc(self::GET_PRODUCT_DETAILS, $params);
			return $detalhesProduto;
		} catch (\Exception $e) {
			return array(
				'error' => true,
				'message' => $e->getMessage());
		}
	}


	const GET_PRODUCT_DETAILS = "
		SELECT PR.CDPRODUTO, PR.CDARVPROD, PR.NMPRODUTO, PR.IDTIPOCOMPPROD
		  FROM PRODUTO PR
		 WHERE PR.CDPRODUTO = :CDPRODUTO
	";

	private function defineRetiradaBalcao(&$dadosPedido){
		if($dadosPedido['ISTOGO'] == true){
			$dadosPedido['IDRETBALLOJA'] = 'S';
		} else if($dadosPedido['ISTOGO'] == false) {
			$dadosPedido['IDRETBALLOJA'] = 'N';
		}
	}

	private function validaDadosComanda($dadosFilial){
		$result = array();
		$retornoVendedorPadrao = self::retornaVendedorPadrao($dadosFilial['CDFILIAL']);

		if($retornoVendedorPadrao['error'] == false){
			$result['data']['CDVENDEDOR'] = $retornoVendedorPadrao['data']['CDVENDEDOR'];
			$retornoOperador = self::retornaOperador($retornoVendedorPadrao['data']);

			if ($retornoOperador['error'] == false) {
				$result['data']['CDOPERADOR'] = $retornoOperador['CDOPERADOR'];
				$retornoIdcontroprod = self::retornaIdcontroprod($dadosFilial);

				if ($retornoIdcontroprod['error'] == false) {
					$result['error'] = false;
					$result['data']['IDCONTROPROD'] = $retornoIdcontroprod['IDCONTROPROD'];
					return $result;
				} else {
					return $retornoIdcontroprod;
				}
			} else {
				return $retornaOperador;	
			}
		} else {
			return $retornoVendedorPadrao;
		}
	}

	private function trataAgendamento($dadosPedido){
		$isScheduled = $dadosPedido['schedule']['TOSCHEDULE'];
		if($isScheduled){
			$scheduleTime = date("H:i:s", strtotime($dadosPedido['schedule']['scheduleData']['time']));
			$DTHRAGENDADA = $dadosPedido['schedule']['scheduleData']['date'] . ' ' . $scheduleTime;
			return array(
				'isScheduled' => $isScheduled,
				'DTHRAGENDADA' => $DTHRAGENDADA);
		} else {
			return array(
				'isScheduled' => $isScheduled,
				'DTHRAGENDADA' => null);
		}
	}

	private function trataSTComanda($arrayScheduled, $validaDados){
		 if ($arrayScheduled['isScheduled']) {
            $IDSTCOMANDA = 'A';
        } else if ($validaDados['IDCONTROPROD'] == 'S'){
            $IDSTCOMANDA = '1';
        } else {
            $IDSTCOMANDA = '3';
        }
        return $IDSTCOMANDA;
	}

	private function abreComanda($validaDados, $dadosFilial, $dadosConsumidor, $dadosPedido, $enderecoComanda, $pagamento){
		try {
			$NRVENDAREST = self::generateVendarestSeq($dadosFilial['CDFILIAL']);
			$NRCOMANDA = self::generateComandavenSeq($dadosFilial['CDFILIAL']);
			$paramsInsertVendaRest = self::preparaInsereVendaRest($dadosFilial, $validaDados, $NRVENDAREST, $dadosConsumidor, $dadosPedido);
			$insereVendaRest = self::insereVendaRest($paramsInsertVendaRest);
			$arrayScheduled = self::trataAgendamento($dadosPedido);
			$IDSTCOMANDA = self::trataSTComanda($arrayScheduled, $validaDados);
			$descricaoCupom = '';
			if(isset($pagamento['promoCoupon'])){
				$descricaoCupom = $pagamento['promoCoupon']['cupom'];
			}
			if($insereVendaRest['error'] == false) {
				$paramsInsertComanda = self::preparaInsereComandaVen($dadosFilial, $dadosConsumidor, $dadosPedido, $NRVENDAREST, $NRCOMANDA, $IDSTCOMANDA, $arrayScheduled, $enderecoComanda, $descricaoCupom);
				$insereComandaVen = self::insereComandaVen($paramsInsertComanda);
				if($insereComandaVen['error'] == false){
					return array('error' => false,
						'dadosComanda' => $paramsInsertComanda);
				} else {
					return $insereComandaVen;
				}
			} else {
				return $insereVendaRest;
			}
		} catch(\Exception $e){
			return array(
				'error' => true,
				'message' => $e->getMessage()
			);
		}
	}

	public function preparaInsereVendaRest($dadosFilial, $validaDados, $NRVENDAREST, $dadosConsumidor, $dadosPedido){
		$params = array(
			'CDFILIAL' => $dadosFilial['CDFILIAL'],
			'NRVENDAREST' => $NRVENDAREST,
			'CDLOJA'=> $dadosFilial['CDLOJA'],
			'NRMESA'=> isset($dadosPedido['NRMESA']) ? $dadosPedido['NRMESA'] : null,
			'CDVENDEDOR' => $validaDados['CDVENDEDOR'],
	        'DTHRABERMESA' => '',
	        'DTHRFECHMESA' => '', 
	        'CDOPERADOR' => $validaDados['CDOPERADOR'],
	        'NRPESMESAVEN' => isset($dadosPedido['NRPESMESAVEN']) ? $dadosPedido['NRPESMESAVEN'] : null,
	        'CDCLIENTE' => $dadosConsumidor['CDCLIENTE'],
	        'CDCONSUMIDOR' => $dadosConsumidor['CDCONSUMIDOR'],
	        'IDPEDIDOPAGO' => 'S');
		return $params;
	}

	public function insereVendaRest($params){
		try {
            if ($this->util->databaseIsOracle()) {
                $this->entityManager->getConnection()->executeQuery(self::ORACLE_INSERT_VENDAREST, $params);
            } else {
                $this->entityManager->getConnection()->executeQuery(self::SQL_INSERT_VENDAREST, $params);
            }
            $return = array(
				'error' => false
			);
			return $return;
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir vendarest. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

	const SQL_INSERT_VENDAREST = "
        INSERT INTO VENDAREST (
            CDFILIAL, NRVENDAREST, CDLOJA, NRMESA, CDVENDEDOR,
            DTHRABERMESA, DTHRFECHMESA, CDOPERADOR, NRPESMESAVEN,
            CDCLIENTE, CDCONSUMIDOR, IDPEDIDOPAGO
        )
        VALUES (
            :CDFILIAL, :NRVENDAREST, :CDLOJA, :NRMESA,
            :CDVENDEDOR, GETDATE(), null, :CDOPERADOR,
            :NRPESMESAVEN, :CDCLIENTE, :CDCONSUMIDOR, :IDPEDIDOPAGO
        )
    ";

    const ORACLE_INSERT_VENDAREST = "
        INSERT INTO VENDAREST (
            CDFILIAL, NRVENDAREST, CDLOJA, NRMESA, CDVENDEDOR,
            DTHRABERMESA, DTHRFECHMESA, CDOPERADOR, NRPESMESAVEN,
            CDCLIENTE, CDCONSUMIDOR, IDPEDIDOPAGO
        )
        VALUES (
            :CDFILIAL, :NRVENDAREST, :CDLOJA, :NRMESA,
            :CDVENDEDOR, SYSDATE, null, :CDOPERADOR,
            :NRPESMESAVEN, :CDCLIENTE, :CDCONSUMIDOR, :IDPEDIDOPAGO
        )
    ";

    public function cancelaPedido($dadosPedido){
    	self::cancelaVendaRest($dadosPedido);
    	self::cancelaComandaVen($dadosPedido['dadosComanda']);
    }

    private function cancelaVendaRest($dadosPedido){
    	$params = array(
    		':CDFILIAL' => $dadosPedido['dadosComanda']['CDFILIAL'],
    		':CDCONSUMIDOR' => $dadosPedido['dadosPagamento'][0]['CDCONSUMIDOR'],
    		':NRVENDAREST' => $dadosPedido['dadosComanda']['NRVENDAREST']
    	);
    	$this->entityManager->getConnection()->executeQuery(self::CANCELA_VENDAREST, $params);
    }
    const CANCELA_VENDAREST = "
	    UPDATE VENDAREST
		SET IDPEDIDOPAGO = 'N'
		WHERE CDFILIAL = :CDFILIAL
		AND CDCONSUMIDOR = :CDCONSUMIDOR
		AND NRVENDAREST = :NRVENDAREST
    ";

    private function cancelaComandaVen($dadosPedido){
    	$params = array(
    		':CDFILIAL' => $dadosPedido['CDFILIAL'],
    		':NRCOMANDA' => $dadosPedido['NRCOMANDA'],
    		':NRVENDAREST' => $dadosPedido['NRVENDAREST']
    	);
    	$this->entityManager->getConnection()->executeQuery(self::CANCELA_COMANDAVEN, $params);
    }
    const CANCELA_COMANDAVEN = "
	    UPDATE COMANDAVEN
		SET IDSTCOMANDA = '4', IDCOMANDAPAGA = 'N'
		WHERE CDFILIAL = :CDFILIAL
		AND NRVENDAREST = :NRVENDAREST
		AND NRCOMANDA = :NRCOMANDA
    ";

	private function preparaInsereComandaVen($dadosFilial, $dadosConsumidor, $dadosPedido, $NRVENDAREST, $NRCOMANDA, $IDSTCOMANDA, $arrayScheduled, $enderecoComanda, $descricaoCupom){
		$CDCAMPANHA = null;
		$VRPONTOBRINDE = null;
		$CDCAMPANHA = null;
		$params = array(
			'CDFILIAL'	=> $dadosFilial['CDFILIAL'],
			'NRVENDAREST'	=> $NRVENDAREST,
			'NRCOMANDA'	=> $NRCOMANDA,
			'CDLOJA'	=> $dadosFilial['CDLOJA'],
			'DSCOMANDA'	=> 'DLV_'.$NRCOMANDA,
			'IDSTCOMANDA'	=>  $IDSTCOMANDA,
			'SGSEXOCON'	=> $dadosConsumidor['IDSEXOCONS'],
			'TXMOTIVCANCE'	=> '',
			'VRACRCOMANDA'	=> 0,
			'CDPAIS'	=> $dadosConsumidor['CDPAIS'],
			'SGESTADO'	=> $dadosConsumidor['SGESTADO'],
			'CDMUNICIPIO'	=> $dadosConsumidor['CDMUNICIPIO'],
			'NRCEPCONSCOMAND'	=> $enderecoComanda['NRCEPCONSCOMAND'],
			'CDBAIRRO'	=> $dadosConsumidor['CDBAIRRO'],
			'DSBAIRRO'	=> $enderecoComanda['DSBAIRRO'],
			'DSENDECONSCOMAN'	=> $enderecoComanda['DSENDECONSCOMAN'],
			'DSCOMPLENDCOCOM'	=> $dadosConsumidor['DSCOMPLENDECONS'],
			'DSREFENDCONSCOM'	=> $dadosConsumidor['DSREFERENDECONS'],
			'IDORGCMDVENDA'	=> '',
			'IDRETBALLOJA'	=> $dadosPedido['IDRETBALLOJA'],
			'NRCOMANDAEXT'	=> '',
			'IDCOMANDAPAGA' => 'S',
			'IDSINCAGENDA' => $arrayScheduled['isScheduled'] ? 'N' : 'S',
            'DTHRAGENDADA' => $arrayScheduled['DTHRAGENDADA'] ? $arrayScheduled['DTHRAGENDADA'] : null,
            'CDCAMPANHA'    => isset($dadosPedido['fidelityData']) ? $dadosPedido['fidelityData']['CDCAMPANHA'] : null,
            'VRPONTOBRINDE' => isset($dadosPedido['fidelityData']) ? $dadosPedido['fidelityData']['VRPONTOBRINDE'] : null,
            'VRDESCFIDELIDAD' => isset($dadosPedido['fidelityData']) ? $dadosPedido['fidelityData']['VALUE'] : null,
            'DSCUPOMPROMO' => $descricaoCupom
        );
        return $params;
	}

	public function insereComandaVen($params){
		try {
			$this->entityManager->getConnection()->executeQuery(self::INSERT_COMANDAVEN, $params);
            $return = array(
				'error' => false
			);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir comandaven. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

	const INSERT_COMANDAVEN = "
        INSERT INTO COMANDAVEN (
            CDFILIAL, NRVENDAREST, NRCOMANDA, CDLOJA,
            DSCOMANDA, IDSTCOMANDA, SGSEXOCON, TXMOTIVCANCE,
            VRACRCOMANDA, CDPAIS, SGESTADO, CDMUNICIPIO,
            NRCEPCONSCOMAND, CDBAIRRO, DSBAIRRO, DSENDECONSCOMAN,
            DSCOMPLENDCOCOM, DSREFENDCONSCOM, IDORGCMDVENDA,
            IDRETBALLOJA, NRCOMANDAEXT, IDSINCAGENDA, DTHRAGENDADA, IDCOMANDAPAGA,
            CDCAMPANHA, VRPONTOBRINDE, VRDESCFIDELIDAD, DSCUPOMPROMO
        )
        VALUES (
            :CDFILIAL, :NRVENDAREST, :NRCOMANDA, :CDLOJA,
            :DSCOMANDA, :IDSTCOMANDA, 'M', :TXMOTIVCANCE,
            :VRACRCOMANDA, :CDPAIS, :SGESTADO, :CDMUNICIPIO,
            :NRCEPCONSCOMAND, :CDBAIRRO, :DSBAIRRO, :DSENDECONSCOMAN,
            :DSCOMPLENDCOCOM, :DSREFENDCONSCOM, :IDORGCMDVENDA,
            :IDRETBALLOJA, :NRCOMANDAEXT, :IDSINCAGENDA, TO_DATE(:DTHRAGENDADA, 'DD/MM/YYYY HH24:MI:SS'), :IDCOMANDAPAGA,
            :CDCAMPANHA, :VRPONTOBRINDE, :VRDESCFIDELIDAD, :DSCUPOMPROMO
        )
    ";

    private function registraPagamento($pagamento, $dadosConsumidor, $dadosFilial, $dadosNovaComanda){
		try {
            $dadosPagamento = $this->trataPedido($pagamento, $dadosConsumidor, $dadosNovaComanda, $dadosFilial);
	        if($dadosPagamento['error'] == false){	        	
		        return array(
		        	'error' => false,
		        	'dadosPagamento' => $dadosPagamento['dadosPagamento']);
	        } else {
	        	return $dadosPagamento;
	        }
		} catch(\Exception $e){
			$return = array(
					'error' => true,
					'message' => $e->getMessage());
		}
    }

    private function trataPedido($arrayPagamento, $dadosConsumidor, $dadosNovaComanda, $dadosFilial){
    	try {
    		$troco = 0;
    		$dadosPagamento = array();
	    	foreach ($arrayPagamento as $pagamento){
	        	if ($pagamento['CDTIPORECE'] == self::TIPO_PAGAMENTO_DINHEIRO){
	                $troco = self::calculaTroco($pagamento);
	            }
	            if ($troco > 0){
	            	$pagamento['TROCO'] = $troco;
	            	$NRSEQMOVDLV = self::geraMovcaixadlvSeq($dadosFilial['CDFILIAL'], $dadosNovaComanda['NRVENDAREST']);
		        	$paramsPagamento = self::preparaInsereMovCaixa($dadosConsumidor, $dadosNovaComanda, $pagamento, $NRSEQMOVDLV, self::MOVIMENTACAO_SAIDA);
		        	$this->insereMovCaixaDlv($paramsPagamento);
		        }
		        $NRSEQMOVDLV = self::geraMovcaixadlvSeq($dadosFilial['CDFILIAL'], $dadosNovaComanda['NRVENDAREST']);
		        $paramsPagamento = self::preparaInsereMovCaixa($dadosConsumidor, $dadosNovaComanda, $pagamento, $NRSEQMOVDLV, self::MOVIMENTACAO_ENTRADA);
	            $this->insereMovCaixaDlv($paramsPagamento);

	            array_push($dadosPagamento, $paramsPagamento);
	        }

            return array(
            	'error' => false,
            	'dadosPagamento' => $dadosPagamento
            );
    	} catch (\Exception $e) {
    		return array(
    			'error' => true,
    			'message' => $e->getMessage());
    	}
    }

    private function calculaTroco($pagamento){
    	$troco = $pagamento['TROCO'] - $pagamento['VALOR'];
        return $troco;
    }

    private function preparaInsereMovCaixa($dadosConsumidor, $dadosNovaComanda, $pagamento, $NRSEQMOVDLV, $TIPO_MOVIMENTACAO){
    	try{
    		if($TIPO_MOVIMENTACAO == self::MOVIMENTACAO_SAIDA){
	    		$VRMOVIVENDDLV = $pagamento['TROCO'];
	    	} else if($TIPO_MOVIMENTACAO == self::MOVIMENTACAO_ENTRADA){
				$VRMOVIVENDDLV = $pagamento['VALOR'];
	    	}
	    	$params = array(
	    		'CDFILIAL' 			=> 		$dadosNovaComanda['CDFILIAL'],
				'NRVENDAREST' 		=>     	$dadosNovaComanda['NRVENDAREST'],
				'NRSEQMOVDLV' 		=>     	$NRSEQMOVDLV,
				'IDTIPOMOVIVEDLV' 	=> 		$TIPO_MOVIMENTACAO,
				'VRMOVIVENDDLV' 	=>   	$VRMOVIVENDDLV,
				'CDTIPORECE' 		=>      $pagamento['CDTIPORECE'],
				'CDCLIENTE' 		=>      $dadosConsumidor['CDCLIENTE'],
				'CDCONSUMIDOR' 		=>    	$dadosConsumidor['CDCONSUMIDOR']
			);
			return $params;
    	} catch(\Exception $e){
    		return array(
    			'error' => true,
    			'message' => $e->getMessage());
    	}
    }

	/**
	 * [retornaOperador description]
	 * @param  [array] $vendedorPadrao [verifica se no vendedor padrão existe um operador na coluna CDOPERADOR]
	 * @return [array] $return [retorna error true caso nao exista operador, ou o cdoperador ]
	 */
	public function retornaOperador($vendedorPadrao){
		try {
			if(!empty($vendedorPadrao['CDOPERADOR'])){
				$return = array(
					'error' => false,
					'CDOPERADOR' => $vendedorPadrao['CDOPERADOR']);
			} else {
				$return = array(
					'error' => true,
					'message' => 'Vendedor padrão da filial não contém operador vinculado.');
			}
			return $return;	
		} catch (Exception $e) {
			$return = array(
					'error' => true,
					'message' => 'Erro ao buscar parametros de vendedor padrao. Erro: '.$e->getMessage());
		}
		
	}

	public function retornaVendedorPadrao($CDFILIAL) {
		$params = array(
			':CDFILIAL'   => $CDFILIAL
		);
		try {
			$standardSeller = $this->entityManager->getConnection()->fetchAssoc(self::GET_STANDARD_SELLER, $params);
			if(!$standardSeller){
				$return = array(
					'error'    => true,
					'message'  => 'Parametrizacao de vendedor padrao incompleta ou inexistente'
				);
			} else {
				$return = array(
					'error' => false,
					'data'  => $standardSeller
				);
			}	
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao buscar parametros de vendedor padrao. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

	const GET_STANDARD_SELLER = "
	    SELECT V.CDVENDEDOR,L.CDLOJA, V.NMFANVEN, C.CDCAIXA, V.CDOPERADOR, L.NRMESAPADRAO AS NRMESA,
	           L.IDPOSOBSPED, L.IDLUGARMESA  
	      FROM PARAVEND PV
	      JOIN VENDEDOR V
	        ON PV.CDVENDEDOR = V.CDVENDEDOR
	      JOIN CAIXA C
	      	ON V.CDCAIXA     = C.CDCAIXA
	       AND PV.CDFILIAL   = C.CDFILIAL
	     JOIN LOJA L
	        ON C.CDLOJA      = L.CDLOJA
	       AND L.CDFILIAL    = C.CDFILIAL 	 
	     WHERE PV.CDFILIAL   = :CDFILIAL
    ";

	public function retornaIdcontroprod($dadosFilial){
		try {
			$params = array(
				':CDFILIAL' => $dadosFilial['CDFILIAL'],
				':CDLOJA' => $dadosFilial['CDLOJA']
			);
            $result = $this->entityManager->getConnection()->fetchAssoc(self::GET_PRODUCT_CONTROL_ID, $params);
            if(!empty($result['IDCONTROPROD'])){
            	$return = array(
            		'error' => false,
            		'IDCONTROPROD' => $result['IDCONTROPROD']);
            } else {
            	$return = array(
            		'error' => true,
            		'message' => 'Controle de produção não definido na loja.');
            }
            return $return;
        } catch (\Exception $e) {
        	$return = array(
				'error'   => true,
				'message' => 'Erro ao buscar parametros de controle de produção. Erro: '.$e->getMessage()
			);
        }
	}

	const GET_PRODUCT_CONTROL_ID = "
		SELECT IDCONTROPROD
		  FROM LOJA
		 WHERE CDFILIAL = :CDFILIAL
		   AND CDLOJA = :CDLOJA
	";

	public function generateVendarestSeq($CDFILIAL){
		$vendarestCounter = 'VENDAREST'.$CDFILIAL;
		$connection 	  = $this->entityManager->getConnection();
		$NRORG    		  = 1;
		$quantity 		  = 1;
		$size     		  = 10;
		$vendarestSeq 	  = $this->util->geraCodigo($connection,$vendarestCounter, $NRORG, $quantity, $size);
		return $vendarestSeq;
	}

	public function generateComandavenSeq($CDFILIAL){
		$comandavenCounter = 'COMANDAVEN'.$CDFILIAL;
		$connection 	   = $this->entityManager->getConnection();
		$NRORG    		   = 1;
		$quantity 		   = 1;
		$size     		   = 10;
		$comandavenSeq 	   = $this->util->geraCodigo($connection,$comandavenCounter, $NRORG, $quantity, $size);
		return $comandavenSeq;
	}

	public function generateItemsSeq($CDFILIAL){
		$itemsCounter      = 'ITCOMANDAVEN'.$CDFILIAL;
        $connection 	   = $this->entityManager->getConnection();
		$NRORG    		   = 1;
		$quantity 		   = 1;
		$size     		   = 10;
		$itemsSeq    	   = $this->util->geraCodigo($connection,$itemsCounter, $NRORG, $quantity, $size);
		return $itemsSeq;
	}

	public function generateItcomandavenSeq($CDFILIAL, $NRCOMANDA){
		$itcomandavenCounter = 'ITCOMANDAVEN'.$CDFILIAL.$NRCOMANDA;
        $connection 	     = $this->entityManager->getConnection();
		$NRORG    		     = 1;
		$quantity 		     = 1;
		$size     		     = 6;
		$NRPRODCOMVEN     = $this->util->geraCodigo($connection,$itcomandavenCounter, $NRORG, $quantity, $size);
		return $NRPRODCOMVEN;
	
	}

	public function geraMovcaixadlvSeq($CDFILIAL, $NRVENDAREST){
		try{
			$movcaixadlvCounter = 'MOVCAIXADLV'.$CDFILIAL.$NRVENDAREST;
			$connection 	    = $this->entityManager->getConnection();
			$NRORG    		    = 1;
			$quantity 		    = 1;
			$size     		    = 10;
			$movcaixadlvSeq     = $this->util->geraCodigo($connection,$movcaixadlvCounter, $NRORG, $quantity, $size);
			return $movcaixadlvSeq;
		} catch(\Exception $e){
			return array('message' => $e->getMessage());
		}
		
	}

	public function insereMovCaixaDlv($params){
		try {
            if ($this->util->databaseIsOracle()) {
                $success = $this->entityManager->getConnection()->executeQuery(self::ORACLE_INSERT_MOVCAIXADLV, $params);
            } else {
                $success = $this->entityManager->getConnection()->executeQuery(self::SQL_INSERT_MOVCAIXADLV, $params);
            }
            $return = array(
				'error' => false
			);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir pagamento. Erro: '.$e->getMessage()
			);
		}

		return $return;
	}

	const ORACLE_INSERT_MOVCAIXADLV = "
        INSERT INTO MOVCAIXADLV (
            CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
            VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR)
        VALUES (
            :CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
            :VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR
        )
    ";

    const SQL_INSERT_MOVCAIXADLV = "
        INSERT INTO MOVCAIXADLV (
            CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
            VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR)
        VALUES (
            :CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
            :VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR
        )
    ";

	public function insertItComandaVen($params){
		try {
            // verifica qual banco de dados está sendo utilizado
            if ($this->util->databaseIsOracle()) {
                $success = $this->entityManager->getConnection()->executeQuery(\Odhen\API\Util\Query::ORACLE_INSERT_ITCOMANDAVEN, $params);
            } else {
                $success = $this->entityManager->getConnection()->executeQuery(\Odhen\API\Util\Query::SQL_INSERT_ITCOMANDAVEN, $params);
            }
            $return = array(
				'error' => false
			);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir produto. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

	public function insertObsItComandaVen($params){
		try {
            // verifica qual banco de dados está sendo utilizado
            $success = $this->entityManager->getConnection()->executeQuery(\Odhen\API\Util\Query::INSERT_OBSITCOMANDAVEN, $params);
            $return = array(
				'error' => false
			);
		} catch (\Exception $e) {
			$return = array(
				'error'   => true,
				'message' => 'Erro ao inserir observacao. Erro: '.$e->getMessage()
			);
		}
		return $return;
	}

	public function generateMovcaixamobSeq($CDFILIAL) {
		$movCounter 	  = 'MOVCAIXAMOB' . $CDFILIAL;
		$connection 	  = $this->entityManager->getConnection();
		$NRORG    		  = 1;
		$quantity 		  = 1;
		$size     		  = 10;
		$movSeq 		  = $this->util->geraCodigo($connection, $movCounter, $NRORG, $quantity, $size);
		return $movSeq;
	}

	/*
	 * Verifica se o IDCHECKIN é válido
	 */
	public function isValidCheckinId($checkinId) {
		$params = array(
			':IDCHECKIN' => $checkinId
		);

		$return = array(
			'error' => false
		);		
		
		try {
			$response = $this->entityManager->getConnection()->fetchAssoc(\Odhen\API\Util\Query::CHECK_VALID_CHECKINID, $params);
			if(!$response) {
				$return['error']	= true;
				$return['message']	= 'CheckinId inválido';
			}	
		} catch (\Exception $e) {
			$return['error']	= true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}

		return $return;
	}
	
	/*
	 * Retorna CDFILIAL, NRVENDAREST
	 */
	public function getResultCheckinMesPos($checkinId) {
		$params = array(
            ':IDCHECKIN' => $checkinId
        );

		$return = array(
			'error' => false
		);		

		try {
			$response = $this->entityManager->getConnection()->fetchAssoc(\Odhen\API\Util\Query::GET_RESULT_CHECKINMESPOS, $params);
			if($response) {
				$return['data'] 	= $response;
			} else {
				$return['error']	= true;
				$return['message']	= 'CDFILIAL e NRVENDAREST não encontrados';
			}	
		} catch (\Exception $e) {
			$return['error']	= true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}

		return $return;
	}

	/*
	 * Retorna CDVENDEDOR, CDCAIXA, CDLOJA
	 */
	public function getResultParavendAndVendedor($CDFILIAL) {
		$params = array(
        	':CDFILIAL' => $CDFILIAL
        );

		$return = array(
			'error' => false
		);		

		try {
			$response = $this->entityManager->getConnection()->fetchAssoc(\Odhen\API\Util\Query::SQL_STANDARD_SELLER, $params);
			if($response) {
				$return['data'] 	= $response;
			} else {
				$return['error']	= true;
				$return['message']	= 'CDVENDEDOR, CDCAIXA e CDLOJA não encontrados';
			}	
		} catch (\Exception $e) {
			$return['error']	= true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}

		return $return;
	}

	/*
	 * Retorna NRCOMANDA
	 */
	public function getNrcomanda($NRVENDAREST, $CDFILIAL) {

		$params = array(
        	':NRVENDAREST'	=> $NRVENDAREST,
            ':CDFILIAL' 	=> $CDFILIAL
        );

		$return = array(
			'error' => false
		);		
		
		try {
			$response = $this->entityManager->getConnection()->fetchAssoc(\Odhen\API\Util\Query::GET_NRCOMANDA, $params);
			if($response) {
				$return['data'] 	= $response;
			} else {
				$return['error']	= true;
				$return['message']	= 'NRCOMANDA não encontrado';
			}	
		} catch (\Exception $e) {
			$return['error']	= true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}

		return $return;
	}

	/*
	 * Retorna NRMESA
	 */
	public function getNrmesa($CDFILIAL, $NRVENDAREST) {
		$params = array(
        	':CDFILIAL' 	=> $CDFILIAL,
            ':NRVENDAREST'	=> $NRVENDAREST
        );

		$return = array(
			'error' => false
		);		
		
		try {
			$response = $this->entityManager->getConnection()->fetchAssoc(\Odhen\API\Util\Query::GET_NRMESA, $params);
			if($response) {
				$return['data'] 	= $response;
			} else {
				$return['error']	= true;
				$return['data'] 	= 'NRMESA não encontrado';
			}	
		} catch (\Exception $e) {
			$return['error']	= true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}

		return $return;
	}


	/*
	 * Insere o pagamento na tabela MOVCAIXAMOB
	 */
	public function insertMovcaixamob($NRSEQMOVMOB, $CDFILIAL, $CDCAIXA, $CDVENDEDOR, $NRVENDAREST, $NRCOMANDA, $NRSEQVENDA, $NRMESA, $NRLUGARMESA, $CDTIPORECE, $IDTIPMOV, $VRMOV, $DSBANDEIRA, $IDTPTEF) {
		$params = array(
	        ':NRSEQMOVMOB'	=> $NRSEQMOVMOB, 
	        ':CDFILIAL' 	=> $CDFILIAL, 
	        ':CDCAIXA'		=> $CDCAIXA, 
	        ':CDVENDEDOR'	=> $CDVENDEDOR, 
	        ':NRVENDAREST'	=> $NRVENDAREST, 
	        ':NRCOMANDA'	=> $NRCOMANDA, 
	        ':NRSEQVENDA'	=> $NRSEQVENDA, 
	        ':NRMESA'		=> $NRMESA,
	        ':NRLUGARMESA'	=> $NRLUGARMESA, 
	        ':CDTIPORECE'	=> $CDTIPORECE, 
	        ':IDTIPMOV' 	=> $IDTIPMOV, 
	        ':VRMOV'		=> $VRMOV, 
	        ':DSBANDEIRA'	=> $DSBANDEIRA, 
	        ':IDTPTEF'		=> $IDTPTEF
        );

		$return = array(
			'error' => false
		);		

		try {
        	$this->entityManager->getConnection()->executeQuery(\Odhen\API\Util\Query::INSERT_MOVCAIXAMOB, $params);
		} catch (\Exception $e) {
			$return['error']	= true;
			$return['message']	= 'Erro: ' . $e->getMessage();
		}

		return $return;
	}
	
	public function changeTableStatus($IDSTMESAAUX, $CDFILIAL, $CDLOJA, $NRMESA) {
		$params = array(
			':IDSTMESAAUX'	=> $IDSTMESAAUX, 
			':CDFILIAL' 	=> $CDFILIAL, 
			':CDLOJA'		=> $CDLOJA, 
			':NRMESA'		=> $NRMESA
		);

		$return = array(
			'error' => false
		);		

		try {
        	$this->entityManager->getConnection()->executeQuery(\Odhen\API\Util\Query::SQL_ALTERA_STATUS_MESA, $params);
		} catch(\Exception $e) {
			$return['error']	= true;
			$return['message']	= $e->getMessage();
		}

		return $return;
	}

    // @toDo: BUSCA VENDA PELO ID EXTERNO

    public function getExternalSaleData($NRCOMANDAEXT) {
	    try {
	        $data   = array(); // dados a serem retornados

	        $return = self::getLastSale($NRCOMANDAEXT);

	        if (!$return['error']) {
	            $lastSale = $return['data'];
	            
	            if (!empty($lastSale)) {
	                // formata os dados da última venda aprovada do cliente
	                $deliveryFee   = $lastSale['IDRETBALLOJA'] == 'S' ? 0 : $lastSale['VRACRCOMANDA'];
	                $addressAsArray = explode(',',$lastSale['DSENDECONSCOMAN']);
	                $streetAddress = $addressAsArray[0] !== "" ? $addressAsArray[0] : null;
	                $number = null;
	                if(count($addressAsArray) > 1){
	                    $number = $addressAsArray[1];
	                }

	                $orderDiscount = $this->util->float($lastSale['VRDESCOMANDA']);
	                $lastSaleData = array(
	                    'CDFILIAL'   => $lastSale['CDFILIAL'],
	                    'CDCLIENTE'       => $lastSale['CDCLIENTE'],
	                    'CDCONSUMIDOR'     => $lastSale['CDCONSUMIDOR'],
	                    'DTVENDA'         => $lastSale['DTVENDA'],
	                    'NRCOMANDA'         => $lastSale['NRCOMANDA'],
	                    'NRVENDAREST'     => $lastSale['NRVENDAREST'],
	                    'IDSTCOMANDA'      => self::status[$lastSale['IDSTCOMANDA']],
	                    'isOpen'           => true,
	                    'PRODUCTSDETAILS'  => array(),
	                    'RECEBIMENTOS'      => array(),
	                    'orderDescription' => array(
	                        'IDRETBALLOJA'     => $lastSale['IDRETBALLOJA'] == 'S' ? 'TGO' : 'DLV',
	                        'productsValue' => 0,
	                        'deliveryFee'   => $deliveryFee,
	                        'orderDiscount' => $orderDiscount,
	                        'totalValue'    => 0,
	                        'description'   => '',
	                        'orderCPF'      => null,
	                        'changeValue'   => 0,
	                        'deliveryAddress' => array(
	                            'streetAddress' => $streetAddress,
	                            'DSBAIRRO'  => $lastSale['DSBAIRRO'],
	                            'number'        => $number,
	                            'DSCOMPLENDCOCOM'  => $lastSale['DSCOMPLENDCOCOM'],
	                            'DSREFENDCONSCOM'     => $lastSale['DSREFENDCONSCOM'],
	                            'CDMUNICIBGE'          => $lastSale['CDMUNICIBGE'],
	                            'SGESTADO'         => $lastSale['SGESTADO'],
	                            'CDPAIS'       => $lastSale['CDPAIS'],
	                            'NRCEPCONSCOMAND'       => $lastSale['NRCEPCONSCOMAND'],
	                            'phone'         => null,
	                            'cellphone'     => null 
	                        )
	                    )
	                );
	                // busca pelos produtos relacionados à venda
	                $return = self::getLastSaleProducts($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA']);
	                if (!$return['error']) {
	                    $products = $return['data'];
	                    // formata dados retornados para cada item da venda
	                    foreach ($products as $product) {
	                        if(!empty($product['NRINSCRCONS'])){
	                            $lastSaleData['orderDescription']['orderCPF'] = $product['NRINSCRCONS'];
	                        }
	                        $VRPRECCOMVEN = $this->util->float($product['VRPRECCOMVEN']);
	                        $VRDESCCOMVEN  = $this->util->float($product['VRDESCCOMVEN']);
	                        $QTPRODCOMVEN  = $this->util->float($product['QTPRODCOMVEN']);
	                        $NRPRODCOMVEN = $product['NRPRODCOMVEN'];

	                        $lastSaleData['orderDescription']['productsValue'] += $VRPRECCOMVEN*$QTPRODCOMVEN - $VRDESCCOMVEN; 

	                        $productData = array(
	                            'CDPRODUTO'        => $product['CDPRODUTO'],
	                            'NMPRODUTO'        => $product['NMPRODUTO'],
	                            'IDIMPPRODUTO'     => $product['IDIMPPRODUTO'],
	                            'QTPRODCOMVEN'     => $QTPRODCOMVEN,
	                            'VRPRECITEM'       => $VRPRECCOMVEN,
	                            'VRDESCCOMVEN'     => $VRDESCCOMVEN,
	                            'OBSERVACOES'      => array()
	                        );
	                        
	                        $return = self::getLastSaleProductObservations($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $NRPRODCOMVEN);
	                        if(!$return['error']){
	                            $observations = $return['data'];
	                            foreach($observations as $observation){
	                                $price = $observation['VRPRECCOMVEN'];
	                                $price = $price == null  ||  $price == 0 ? null : $this->util->float($price);
	                                
	                                if($price !== null){
	                                    $lastSaleData['orderDescription']['productsValue'] += $price;
	                                }
	                                $observationData = array(
	                                    'CDGRPOCOR'    => $observation['CDGRPOCOR'],
	                                    'CDOCORR'      => $observation['CDOCORR'],
	                                    'DSOCORR'      => $observation['DSOCORR'],
	                                    'quantity'     => 1,    
	                                    'price'        => $price    
	                                );
	                                
	                                array_push($productData['OBSERVACOES'], $observationData);
	                            }
	                        }
	                        array_push($lastSaleData['PRODUCTSDETAILS'], $productData);
	                    }
	                }

	                // busca pelos produtos combo relacionados à venda
	                $return = self::getLastSaleCombos($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA']);
	                
	                if (!$return['error']) {
	                    $combos = $return['data'];
	                    foreach ($combos as $combo) {
	                        $comboData = array(
	                            'productCode'   => $combo['CDPRODPROMOCAO'],
	                            'productName'   => $combo['NMPRODUTO'],
	                            'quantity'      => 1, 
	                            'unitValue'     => $this->util->float($combo['VRPRECCOMVEN']),
	                            'discount'      => $this->util->float($combo['VRDESCCOMVEN']),
	                            'observations'  => array(),
	                            'childItems'    => array()
	                        );
	                        
	                        $return = self::getLastSaleComboProducts($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $combo['CDPRODPROMOCAO'], $combo['NRSEQPRODCOM']);
	                        if (!$return['error']) {
	                            $comboProducts = $return['data'];
	                            
	                            $comboName = array();
	                            $comboDescription = array();
	                            
	                            foreach ($comboProducts as $comboProduct) {
	                                if(!empty($comboProduct['NRINSCRCONS'])){
	                                    $lastSaleData['orderDescription']['orderCPF'] = $comboProduct['NRINSCRCONS'];
	                                }
	                                $VRPRECCOMVEN = $this->util->float($comboProduct['VRPRECCOMVEN']);
	                                $VRDESCCOMVEN  = $this->util->float($comboProduct['VRDESCCOMVEN']);
	                                $QTPRODCOMVEN  = $this->util->float($comboProduct['QTPRODCOMVEN']);
	                                $NRPRODCOMVEN = $comboProduct['NRPRODCOMVEN'];

	                                $lastSaleData['orderDescription']['productsValue'] += $VRPRECCOMVEN*$QTPRODCOMVEN - $VRDESCCOMVEN; 
	                               
	                                $comboProductData = array(
	                                    'productCode'   => $comboProduct['CDPRODUTO'],
	                                    'productName'   => $comboProduct['NMPRODUTO'],
	                                    'quantity'      => $QTPRODCOMVEN, 
	                                    'unitValue'     => $VRPRECCOMVEN,
	                                    'discount'      => $VRDESCCOMVEN,
	                                    'observations'  => array()
	                                );

	                                $return = self::getLastSaleProductObservations($lastSale['CDFILIAL'], $lastSale['NRVENDAREST'], $lastSale['NRCOMANDA'], $NRPRODCOMVEN);
	                                if(!$return['error']){
	                                    $observations = $return['data'];
	                                    foreach($observations as $observation){
	                                        $price = $observation['VRPRECCOMVEN'];
	                                        $price = $price == null  ||  $price == 0 ? null : $this->util->float($price);
	                                        if($price !== null){
	                                            $lastSaleData['orderDescription']['productsValue'] += $price;
	                                        }
	                                        $observationData = array(
	                                            'obsGroupCode' => $observation['CDGRPOCOR'],
	                                            'obsCode'      => $observation['CDOCORR'],
	                                            'obsName'      => $observation['DSOCORR'],
	                                            'quantity'     => 1,    
	                                            'price'        => $price     
	                                        );
	                                        array_push($comboProductData['observations'], $observationData);
	                                    }
	                                }
	                                
	                                array_push($comboData['childItems'], $comboProductData);
	                            }

	                            array_push($lastSaleData['PRODUCTSDETAILS'], $comboData);
	                        }
	                    }
	                }

	                // busca pelas formas de pagamento relacionadas à venda
	                $return = self::getLastSalePayments($lastSale['CDFILIAL'], $lastSale['NRVENDAREST']);
	                if (!$return['error']) {
	                    $payments = $return['data'];
	                    
	                    // formata dados retornados para cada pagamento
	                    foreach ($payments as $payment) {
	                        $paymentData = array(
	                            'CDTIPORECE' => $payment['CDTIPORECE'],
	                            'NMTIPORECE' => $payment['NMTIPORECE'],
	                            'VALOR'      => $this->util->float($payment['VRMOVIVEND'])
	                        );
	                        
	                        array_push($lastSaleData['RECEBIMENTOS'], $paymentData);
	                    }
	                } 
	                $orderDescription = &$lastSaleData['orderDescription'];
	                $orderDescription['totalValue'] = $orderDescription['deliveryFee'] + $orderDescription['productsValue'] - $orderDescription['orderDiscount'];  

	                $data = array($lastSaleData);
	                return array(
	                	'error' => false,
	                	'data' => $data
	                );
	            } else {
	            	return array(
	                	'error' => true,
	                	'message' => 'Nenhuma venda foi encontrada.'
	                );
	            }
	        } else {
	        	return array(
                	'error' => true,
                	'message' => $return['message']
                );
	        }
	    } catch (\Exception $e) {
	    	return array(
            	'error' => true,
            	'message' => 'Erro ao buscar última venda do consumidor. ' . $e->getMessage()
            );
	    }
	}

	public function getLastSale($NRCOMANDAEXT) {
        $params = array(
            ':NRCOMANDAEXT'    => $NRCOMANDAEXT
        );
        
        $errorMessage = 'Erro ao buscar a venda.';
        try {
            // verifica qual banco de dados está sendo utilizado
            if ($this->util->databaseIsOracle()) {
                $result = $this->entityManager->getConnection()->fetchAssoc(self::ORACLE_GET_LAST_SALE, $params);
            } else {
                $result = $this->entityManager->getConnection()->fetchAssoc(self::SQL_GET_LAST_SALE, $params);
            }

            if($result) {
                $return = array(
                    'error' => false,
                    'data'  => $result
                );
            } else {
                $return = array(
                    'error'   => true,
                    'message' => 'Venda não encontrada.'
                );
            }
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => $errorMessage.'. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    const ORACLE_GET_LAST_SALE = "
	    SELECT VR.CDCLIENTE, VR.CDCONSUMIDOR, VR.CDFILIAL, CV.NRCOMANDA, VR.NRVENDAREST,
               TO_CHAR(VR.DTHRABERMESA, 'DD/MM/YYYY HH24:MI') AS DTVENDA, 
               CV.IDSTCOMANDA, CV.IDRETBALLOJA, CV.VRACRCOMANDA, CV.VRDESCOMANDA, 
               M.CDMUNICIBGE, 'BR' CDPAIS , CV.SGESTADO, CV.DSENDECONSCOMAN, CV.DSCOMPLENDCOCOM, CV.NRCEPCONSCOMAND,
               CV.DSREFENDCONSCOM, CV.DSBAIRRO,
               CASE CV.IDSTCOMANDA
                    WHEN '1' THEN 1 -- em aberto
                    ELSE 0          -- fechado
               END AS ISOPEN
           
		  FROM COMANDAVEN CV
	      JOIN VENDAREST VR
	        ON CV.CDFILIAL    = VR.CDFILIAL
	       AND CV.NRVENDAREST = VR.NRVENDAREST
	       
	 LEFT JOIN MUNICIPIO M 
	 		ON CV.CDMUNICIPIO = M.CDMUNICIPIO
	 	   AND CV.SGESTADO    = M.SGESTADO
	 	   AND CV.CDPAIS      = M.CDPAIS


		 WHERE CV.NRCOMANDAEXT = :NRCOMANDAEXT
  
	  ORDER BY VR.DTHRABERMESA DESC
	";
	
	const SQL_GET_LAST_SALE = "
		SELECT VR.CDCLIENTE, VR.CDCONSUMIDOR, VR.CDFILIAL, CV.NRCOMANDA, VR.NRVENDAREST,
               CONVERT(VARCHAR(12), VR.DTHRABERMESA, 104) AS DTVENDA, 
               CV.IDSTCOMANDA, CV.IDRETBALLOJA, CV.VRACRCOMANDA, CV.VRDESCOMANDA, 
               M.CDMUNICIBGE, 'BR' CDPAIS , CV.SGESTADO, CV.DSENDECONSCOMAN, CV.DSCOMPLENDCOCOM, CV.NRCEPCONSCOMAND,
               CV.DSREFENDCONSCOM, CV.DSBAIRRO,
               CASE CV.IDSTCOMANDA
                    WHEN '1' THEN 1 -- em aberto
                    ELSE 0          -- fechado
               END AS ISOPEN
           
		  FROM COMANDAVEN CV
	      JOIN VENDAREST VR
	        ON CV.CDFILIAL    = VR.CDFILIAL
	       AND CV.NRVENDAREST = VR.NRVENDAREST
	       
	 LEFT JOIN MUNICIPIO M 
	 		ON CV.CDMUNICIPIO = M.CDMUNICIPIO
	 	   AND CV.SGESTADO    = M.SGESTADO
	 	   AND CV.CDPAIS      = M.CDPAIS


		 WHERE CV.NRCOMANDAEXT = :NRCOMANDAEXT
  
	  ORDER BY VR.DTHRABERMESA DESC
	";

    // --- BUSCA PRODUTOS
    public function getLastSaleProducts($CDFILIAL, $NRVENDAREST, $NRCOMANDA) {
        $params = array(
            ':CDFILIAL'    => $CDFILIAL, 
            ':NRVENDAREST' => $NRVENDAREST, 
            ':NRCOMANDA'   => $NRCOMANDA    
        );
        
        try {
            // verifica qual banco de dados está sendo utilizado
            if ($this->util->databaseIsOracle()) {
                $products = $this->entityManager->getConnection()->fetchAll(self::ORACLE_GET_LASTSALE_PRODUCTS, $params);
            } else {
                $products = $this->entityManager->getConnection()->fetchAll(self::SQL_GET_LASTSALE_PRODUCTS, $params);
            }
            $return = array(
                'error' => false,
                'data'  => $products
            );
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao buscar produtos relacionados à venda '.$NRCOMANDA.'. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    const ORACLE_GET_LASTSALE_PRODUCTS = "
		SELECT IT.CDPRODUTO, PR.NMPRODUTO, NVL(PR.DSPRODVENDA, '') AS DESCRIPTION, '' AS CDPRODPROMOCAO, 
			   IT.VRPRECCOMVEN , IT.QTPRODCOMVEN , IT.VRDESCCOMVEN, IT.VRACRCOMVEN, IT.NRINSCRCONS, IT.NRPRODCOMVEN
			   
		  FROM ITCOMANDAVEN IT
		  JOIN PRODUTO PR
		    ON IT.CDPRODUTO = PR.CDPRODUTO
		  
		 WHERE IT.CDFILIAL     = :CDFILIAL
		   AND IT.NRVENDAREST  = :NRVENDAREST
           AND IT.NRCOMANDA    = :NRCOMANDA
           AND IT.CDPRODUTO NOT IN (SELECT NVL(CDPRODUTO,'VAZIO') FROM OCORRENCIA)
		   AND IT.CDPRODPROMOCAO IS NULL
	";
	
	const SQL_GET_LASTSALE_PRODUCTS = "
		SELECT IT.CDPRODUTO, PR.NMPRODUTO, ISNULL(PR.DSPRODVENDA, '') AS DESCRIPTION, '' AS CDPRODPROMOCAO, 
			   IT.VRPRECCOMVEN , IT.QTPRODCOMVEN , IT.VRDESCCOMVEN, IT.VRACRCOMVEN, 
			   IT.NRINSCRCONS, IT.NRPRODCOMVEN, PR.IDIMPPRODUTO
		  FROM ITCOMANDAVEN IT
		  JOIN PRODUTO PR
		    ON IT.CDPRODUTO = PR.CDPRODUTO
		  
		 WHERE IT.CDFILIAL     = :CDFILIAL
		   AND IT.NRVENDAREST  = :NRVENDAREST
           AND IT.NRCOMANDA = :NRCOMANDA
           AND IT.CDPRODUTO NOT IN (SELECT ISNULL(CDPRODUTO,'VAZIO') FROM OCORRENCIA)
		   AND IT.CDPRODPROMOCAO IS NULL
	";

    public function getLastSaleProductObservations($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN){
        $params = array(
            ':CDFILIAL'    => $CDFILIAL, 
            ':NRVENDAREST' => $NRVENDAREST, 
            ':NRCOMANDA'   => $NRCOMANDA,
            ':NRPRODCOMVEN' => $NRPRODCOMVEN    
        );
        
        try {
            if ($this->util->databaseIsOracle()) {
                $observations = $this->entityManager->getConnection()->fetchAll(self::GET_LASTSALE_PRODUCT_OBSERVATIONS, $params);
            } else {
                $observations = $this->entityManager->getConnection()->fetchAll(self::GET_POSITION_PRODUCT_OBSERVATIONS, $params);
            }
      
            $return = array(
                'error' => false,
                'data'  => $observations
            );

        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao buscar observacoes. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    const GET_LASTSALE_PRODUCT_OBSERVATIONS = " 	
	  SELECT DSOCORR, CDOCORR, CDGRPOCOR, VRPRECCOMVEN, CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN
	    FROM (
	    	SELECT NVL(OC.DSAPELIDOOBS,OC.DSOCORR) DSOCORR, OC.CDOCORR, OC.CDGRPOCOR, 0 VRPRECCOMVEN,
	    		   OBIT.CDFILIAL, OBIT.NRVENDAREST, OBIT.NRCOMANDA, OBIT.NRPRODCOMVEN	
		
			  FROM OBSITCOMANDAVEN OBIT
	  		  JOIN OCORRENCIA OC
		    	ON OBIT.CDGRPOCOR = OC.CDGRPOCOR
		   	   AND OBIT.CDOCORR   = OC.CDOCORR 	  
		
	     	 WHERE OC.CDPRODUTO IS NULL

		     UNION 

		   	SELECT NVL(OC.DSAPELIDOOBS,OC.DSOCORR) DSOCORR, OC.CDOCORR, OC.CDGRPOCOR, NVL(IT.VRPRECCOMVEN,0) VRPRECCOMVEN ,
		   		   OBIT.CDFILIAL, OBIT.NRVENDAREST, OBIT.NRCOMANDA, OBIT.NRPRODCOMVEN 
		
			  FROM OBSITCOMANDAVEN OBIT
			  JOIN OCORRENCIA OC
		    	ON OBIT.CDGRPOCOR = OC.CDGRPOCOR
		   	   AND OBIT.CDOCORR   = OC.CDOCORR	
         LEFT JOIN (SELECT DISTINCT CDFILIAL, NRCOMANDA, NRVENDAREST, CDPRODUTO, VRPRECCOMVEN
         				       FROM ITCOMANDAVEN 
         			)IT
		        ON IT.NRCOMANDA    = OBIT.NRCOMANDA
		       AND IT.NRVENDAREST  = OBIT.NRVENDAREST
		       AND IT.CDPRODUTO    = OC.CDPRODUTO
		   	   AND IT.CDFILIAL     = OBIT.CDFILIAL 	
		 		   
		   	 WHERE OC.CDPRODUTO IS NOT NULL	
		  )

		  WHERE NRCOMANDA    = :NRCOMANDA
			AND NRVENDAREST  = :NRVENDAREST
			AND CDFILIAL     = :CDFILIAL
		   	AND NRPRODCOMVEN = :NRPRODCOMVEN
	";

    const GET_POSITION_PRODUCT_OBSERVATIONS = " 	
    	SELECT OC.DSOCORR, OC.CDOCORR, OC.CDGRPOCOR, 0 VRPRECCOMVEN,
	    		   OBIT.CDFILIAL, OBIT.NRVENDAREST, OBIT.NRCOMANDA, OBIT.NRPRODCOMVEN	
		
			  FROM OBSITCOMANDAVEN OBIT
	  		  JOIN OCORRENCIA OC
		    	ON OBIT.CDGRPOCOR = OC.CDGRPOCOR
		   	   AND OBIT.CDOCORR   = OC.CDOCORR 	  
		
	     	 WHERE OC.CDPRODUTO IS NULL
	     	   AND OBIT.NRCOMANDA    = :NRCOMANDA
			   AND OBIT.NRVENDAREST  = :NRVENDAREST
		 	   AND OBIT.CDFILIAL     = :CDFILIAL
		   	   AND OBIT.NRPRODCOMVEN = :NRPRODCOMVEN

		     UNION 

		   	SELECT OC.DSOCORR, OC.CDOCORR, OC.CDGRPOCOR, ISNULL(IT.VRPRECCOMVEN,0) VRPRECCOMVEN ,
		   		   OBIT.CDFILIAL, OBIT.NRVENDAREST, OBIT.NRCOMANDA, OBIT.NRPRODCOMVEN 
		
			  FROM OBSITCOMANDAVEN OBIT
			  JOIN OCORRENCIA OC
		    	ON OBIT.CDGRPOCOR = OC.CDGRPOCOR
		   	   AND OBIT.CDOCORR   = OC.CDOCORR	
         LEFT JOIN (SELECT DISTINCT CDFILIAL, NRCOMANDA, NRVENDAREST, CDPRODUTO, VRPRECCOMVEN
         				       FROM ITCOMANDAVEN 
         			)IT
		        ON IT.NRCOMANDA    = OBIT.NRCOMANDA
		       AND IT.NRVENDAREST  = OBIT.NRVENDAREST
		       AND IT.CDPRODUTO    = OC.CDPRODUTO
		   	   AND IT.CDFILIAL     = OBIT.CDFILIAL 	
		 		   
		   	 WHERE OC.CDPRODUTO IS NOT NULL	
		       AND OBIT.NRCOMANDA    = :NRCOMANDA
			   AND OBIT.NRVENDAREST  = :NRVENDAREST
		 	   AND OBIT.CDFILIAL     = :CDFILIAL
		   	   AND OBIT.NRPRODCOMVEN = :NRPRODCOMVEN
	";
    // --- BUSCA COMBOS
    public function getLastSaleCombos($CDFILIAL, $NRVENDAREST, $NRCOMANDA) {
        $params = array(
            ':CDFILIAL'    => $CDFILIAL, 
            ':NRVENDAREST' => $NRVENDAREST, 
            ':NRCOMANDA'   => $NRCOMANDA    
        );
        
        try {
        	if ($this->util->databaseIsOracle()) {
            	$combos = $this->entityManager->getConnection()->fetchAll(self::GET_LASTSALE_COMBOS, $params);
        	} else {
        		$combos = $this->entityManager->getConnection()->fetchAll(self::SQL_GET_LASTSALE_COMBOS, $params);
        	}
            $return = array(
                'error' => false,
                'data'  => $combos
            );
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao buscar combos relacionados à venda. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    const GET_LASTSALE_COMBOS = "
		SELECT IT.CDPRODPROMOCAO AS CDPRODUTO, IT.CDPRODPROMOCAO, PR.NMPRODUTO, IT.NRSEQPRODCOM,
			   SUM(IT.VRPRECCOMVEN) VRPRECCOMVEN, SUM(IT.VRDESCCOMVEN) VRDESCCOMVEN, SUM(IT.VRACRCOMVEN) VRACRCOMVEN
			   
		  FROM ITCOMANDAVEN IT
		  JOIN PRODUTO PR
		    ON IT.CDPRODPROMOCAO = PR.CDPRODUTO
		  
		 WHERE IT.CDFILIAL    = :CDFILIAL
		   AND IT.NRVENDAREST = :NRVENDAREST
           AND IT.NRCOMANDA   = :NRCOMANDA
		   
    	   AND IT.CDPRODPROMOCAO IS NOT NULL
       
	  GROUP BY IT.CDFILIAL, IT.NRVENDAREST, IT.NRCOMANDA, IT.CDPRODPROMOCAO, PR.NMPRODUTO, IT.NRSEQPRODCOM
	";

	const SQL_GET_LASTSALE_COMBOS = "
		SELECT IT.CDPRODPROMOCAO AS CDPRODUTO, IT.CDPRODPROMOCAO, PR.NMPRODUTO, IT.NRSEQPRODCOM,
			   SUM(IT.VRPRECCOMVEN) VRPRECCOMVEN, SUM(IT.VRDESCCOMVEN) VRDESCCOMVEN, SUM(IT.VRACRCOMVEN) VRACRCOMVEN
			   
		  FROM ITCOMANDAVEN IT
		  JOIN PRODUTO PR
		    ON IT.CDPRODPROMOCAO = PR.CDPRODUTO
		  
		 WHERE IT.CDFILIAL    = :CDFILIAL
		   AND IT.NRVENDAREST = :NRVENDAREST
           AND IT.NRCOMANDA   = :NRCOMANDA
		   
    	   AND IT.CDPRODPROMOCAO IS NOT NULL
       
	  GROUP BY IT.CDFILIAL, IT.NRVENDAREST, IT.NRCOMANDA, IT.CDPRODPROMOCAO, PR.NMPRODUTO, IT.NRSEQPRODCOM
	";
    
    // ---- BUSCA PRODUTOS DOS COMBOS
    public function getLastSaleComboProducts($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $CDPRODPROMOCAO, $NRSEQPRODCOM) {
        $params = array(
            ':CDFILIAL'       => $CDFILIAL, 
            ':NRVENDAREST'    => $NRVENDAREST, 
            ':NRCOMANDA'      => $NRCOMANDA,
            ':CDPRODPROMOCAO' => $CDPRODPROMOCAO,
            ':NRSEQPRODCOM'   => $NRSEQPRODCOM
        );
        
        try {
            // verifica qual banco de dados está sendo utilizado
            if ($this->util->databaseIsOracle()) {
                $comboProducts = $this->entityManager->getConnection()->fetchAll(self::ORACLE_GET_LASTSALE_COMBO_PRODUCTS, $params);
            } else {
                $comboProducts = $this->entityManager->getConnection()->fetchAll(self::SQL_GET_POSITION_COMBO_PRODUCTS, $params);
            }
            $return = array(
                'error' => false,
                'data'  => $comboProducts
            );
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao buscar produtos dos combos relacionados à venda'.$e->getMessage()
            );
        }
        
        return $return;
    }

    const ORACLE_GET_LASTSALE_COMBO_PRODUCTS = "
		SELECT IT.CDPRODUTO, PR.NMPRODUTO, NVL(PR.DSPRODVENDA, '') AS DESCRIPTION,
			   IT.VRPRECCOMVEN , IT.QTPRODCOMVEN , IT.VRDESCCOMVEN, IT.VRACRCOMVEN, IT.NRINSCRCONS, IT.NRPRODCOMVEN
			   
		  FROM ITCOMANDAVEN IT
		  JOIN PRODUTO PR
		    ON IT.CDPRODUTO = PR.CDPRODUTO
		  
		 WHERE IT.CDFILIAL       = :CDFILIAL
		   AND IT.NRVENDAREST    = :NRVENDAREST
           AND IT.NRCOMANDA      = :NRCOMANDA
		   AND IT.CDPRODPROMOCAO = :CDPRODPROMOCAO
		   AND IT.NRSEQPRODCOM   = :NRSEQPRODCOM
	";

	const SQL_GET_POSITION_COMBO_PRODUCTS = "
		SELECT IT.CDPRODUTO, PR.NMPRODUTO, ISNULL(PR.DSPRODVENDA, '') AS DESCRIPTION,
			   IT.VRPRECCOMVEN , IT.QTPRODCOMVEN , IT.VRDESCCOMVEN, IT.VRACRCOMVEN, IT.NRINSCRCONS, IT.NRPRODCOMVEN
			   
		  FROM ITCOMANDAVEN IT
		  JOIN PRODUTO PR
		    ON IT.CDPRODUTO = PR.CDPRODUTO
		  
		 WHERE IT.CDFILIAL       = :CDFILIAL
		   AND IT.NRVENDAREST    = :NRVENDAREST
           AND IT.NRCOMANDA      = :NRCOMANDA
		   AND IT.CDPRODPROMOCAO = :CDPRODPROMOCAO
		   AND IT.NRSEQPRODCOM   = :NRSEQPRODCOM
	";

    // --- BUSCA PAGAMENTOS 
    public function getLastSalePayments($CDFILIAL, $NRVENDAREST) {
        $params = array(
            ':CDFILIAL'    => $CDFILIAL, 
            ':NRVENDAREST' => $NRVENDAREST
        );
        
        try {
        	if ($this->util->databaseIsOracle()) {
            	$payments = $this->entityManager->getConnection()->fetchAll(self::GET_LASTSALE_PAYMENTS, $params);
        	} else {
        		$payments = $this->entityManager->getConnection()->fetchAll(self::SQL_GET_LASTSALE_PAYMENTS, $params);
        	}
            $return = array(
                'error' => false,
                'data'  => $payments
            );
        } catch (\Exception $e) {
            $return = array(
                'error'   => true,
                'message' => 'Erro ao buscar formas de pagamento relacionadas à venda '.$NRSEQVENDA.'. Erro: '.$e->getMessage()
            );
        }
        
        return $return;
    }

    const GET_LASTSALE_PAYMENTS = "
		SELECT MV.CDTIPORECE, TR.NMTIPORECE, MV.VRMOVIVENDDLV AS VRMOVIVEND
		  FROM MOVCAIXADLV MV
    	  JOIN TIPORECE TR
            ON MV.CDTIPORECE = TR.CDTIPORECE
      
		 WHERE MV.CDFILIAL    = :CDFILIAL
		   AND MV.NRVENDAREST = :NRVENDAREST
		   AND MV.IDTIPOMOVIVEDLV = 'E'
	";

	const SQL_GET_LASTSALE_PAYMENTS = "
		SELECT MV.CDTIPORECE, TR.NMTIPORECE, MV.VRMOVIVENDDLV AS VRMOVIVEND
		  FROM MOVCAIXADLV MV
    	  JOIN TIPORECE TR
            ON MV.CDTIPORECE = TR.CDTIPORECE
      
		 WHERE MV.CDFILIAL    = :CDFILIAL
		   AND MV.NRVENDAREST = :NRVENDAREST
		   AND MV.IDTIPOMOVIVEDLV = 'E'
	";

}