<?php

namespace Odhen\API\Service;

class TEF {

	const SALVA_LOGS = false;
	const IS_HOMOLOGATION = false;
    const DELPHI_DLL = false;

    protected $entityManager;
    protected $impressaoUtil;
    protected $util;

    public function __construct(
            \Doctrine\ORM\EntityManager $entityManager, 
            \Odhen\API\Lib\ImpressaoUtil $impressaoUtil, 
            \Odhen\API\Util\Util $util,
            \Odhen\API\Service\SiTefPinpad $sitefPinpad,
            \Odhen\API\Service\CapptaPinpad $capptaPinpad) {
        $this->entityManager = $entityManager;
        $this->impressaoUtil = $impressaoUtil;
        $this->util          = $util;
        $this->sitefPinpad   = $sitefPinpad;
        $this->capptaPinpad  = $capptaPinpad;
    }

    /* @toDo: Separate this in a "ImpressaoTEF" Service */
    /********************* IMPRESSAO TEF ********************/

    public function imprimeCupomTEF($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $useClientSiTef, $comprovanteTef) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRORG' => $NRORG
        );

        $dadosImpressora = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_IMPRESSORA, $params);

        if (!empty($dadosImpressora)) {
            $IDMODEIMPRES = $dadosImpressora[0]['IDMODEIMPRES'];
            $CDPORTAIMPR = $dadosImpressora[0]['CDPORTAIMPR'];

            $retorno = $this->impressaoUtil->iniciarPorta($IDMODEIMPRES, $CDPORTAIMPR);
            if ($retorno['error'] == false) {
                if ($useClientSiTef) {
                    $retorno = self::carregaDadosImpressaoTEF($IDMODEIMPRES, $NRSEQVENDA);
                } else {
                    // TEF Data from dll
                    $retorno = array(
                        'error' => false,
                        'dados' => $comprovanteTef
                    );
                }

                if ($retorno['error'] == false) {
                    $printerParams = $this->impressaoUtil->buscaParametrosImpressora($IDMODEIMPRES);

                    // Build tef text structure to print
                    $text = $printerParams['comandoEnter'] . $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
                    $text .= $this->impressaoUtil->imprimeLinha($printerParams, $text) . $printerParams['comandoEnter'];
                    $text .= $retorno['dados'];
                    $text .= $printerParams['comandoEnter'] . $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
                    $text .= $this->impressaoUtil->imprimeLinha($printerParams, $text) . $printerParams['comandoEnter'];

                    $retornoInicia = $this->impressaoUtil->iniciarPorta($IDMODEIMPRES, $CDPORTAIMPR);
                    $this->impressaoUtil->imprimeTexto($text, $IDMODEIMPRES, $retornoInicia['portaUsbKiosk']);
                    $this->impressaoUtil->cortaPapel($IDMODEIMPRES, $retornoInicia['portaUsbKiosk']);
                    $this->impressaoUtil->fechaPorta($IDMODEIMPRES, $retornoInicia['portaUsbKiosk']);
                    $retorno = array(
                        'error' => false
                    );
                }
            }
        } else {
            $retorno = array(
                'error' => true,
                'message' => 'Impressora SAT não parametrizada para o caixa.'
            );
        }

        return $retorno;
    }

    private function carregaDadosImpressaoTEF($IDMODEIMPRES, $NRSEQVENDA) {
        $filePath = $this->processedPath . '\\IntPos.' . $NRSEQVENDA;

        if (file_exists($filePath)) {
            $impFile = fopen($filePath, 'r');
            $dadosArquivo = fread($impFile, filesize($filePath));

            $dadosArquivo = self::indexaArrayDoArquivo($dadosArquivo);

            $corpoImpressao = self::montaDadosImpressaoTEF($IDMODEIMPRES, $dadosArquivo['029'], $dadosArquivo['028-001']);

            fclose($impFile);

            return array(
                'error' => false,
                'dados' => $corpoImpressao
            );
        } else {
            return array(
                'error' => true,
                'message' => 'Não foi possível abrir o arquivo da transação TEF para impressão do comprovante.'
            );
        }
    }

    private function montaDadosImpressaoTEF($IDMODEIMPRES, $dadosImpressao, $qtdLinhas) {
        $corpoImpressao = "";
        $printerParams = $this->impressaoUtil->buscaParametrosImpressora($IDMODEIMPRES);

        $corpoImpressao .= $printerParams['comandoEnter'];

        foreach ($dadosImpressao as $index => $linhaImpressao) {
            if (floatval($index) <= $qtdLinhas) {
                $linhaImpressao = str_replace('"', '', $linhaImpressao);
                $linhaImpressao = $this->impressaoUtil->centraliza($printerParams, $linhaImpressao);
                $corpoImpressao .= $linhaImpressao . $printerParams['comandoEnter'];
            }
        }

        return $corpoImpressao;
    }

    const BUSCA_DADOS_IMPRESSORA = "
        SELECT P.NRSEQIMPRLOJA, I.IDMODEIMPRES, P.CDPORTAIMPR
          FROM CAIXA C JOIN IMPRLOJA P
                         ON C.CDFILIAL = P.CDFILIAL
                        AND C.CDLOJA = P.CDLOJA
                        AND C.NRSEQIMPRLOJA3 = P.NRSEQIMPRLOJA
                       JOIN IMPRESSORA I
                         ON P.CDIMPRESSORA = I.CDIMPRESSORA
         WHERE C.CDFILIAL = :CDFILIAL
           AND C.CDCAIXA = :CDCAIXA
           AND C.NRORG = :NRORG
    ";

    /********************* FIM IMPRESSAO TEF ********************/

    /********************* TRANSAÇÃO TEF DLL ********************/

    public function configuraTransacaoTefDll($IDTPTEF, $CDFILIAL, $CDCAIXA, $NRINSJURFILI, $CDTERTEF) {
        try {
            switch ($IDTPTEF) {
                case '0':
                    $retornoConfigura = $this->sitefPinpad->configuraSitef($CDFILIAL, $CDCAIXA);
                    break;
                case '2':
                    $retornoConfigura = $this->capptaPinpad->configuraCappta($NRINSJURFILI, $CDTERTEF);
                    break;
                default:
                    $retornoConfigura = array(
                        'error'               => true,
                        'finishedTransaction' => true,
                        'message'             => 'Modo TEF não encontrado. Configura a parametrização. (Coluna IDTPTEF).' 
                    );
                    break;
            }
            return $retornoConfigura;
        } catch (\Exception $e) {
            return array(
                'error' => true,
                'message' => 'Mensagem: ' . $e->getMessage()
            );
        }
    }

    public function iniciaTransacaoTefDll($IDTPTEF, $IDTIPORECE, $totalSale, $CDOPERADOR) {
        try {
            switch ($IDTPTEF) {
                case '0':
                    $retornoInicia = $this->sitefPinpad->iniciaSiTef($IDTIPORECE, $totalSale, $CDOPERADOR);
                    break;
                case '2':
                    $retornoInicia = $this->capptaPinpad->iniciaCappta($IDTIPORECE, $totalSale);
                    break;
                default:
                    $retornoInicia = array(
                        'error'               => true,
                        'finishedTransaction' => true,
                        'message'             => 'Modo TEF não encontrado. Confira a parametrização. (Coluna IDTPTEF).' 
                    );
                    break;
            }
            return $retornoInicia;
        } catch (\Exception $e) {
            return array(
                'error' => true,
                'message' => 'Mensagem: ' . $e->getMessage()
            );
        }
    }

    public function continuaTransacaoTefDll($IDTPTEF, $valorTotal, $IDTIPORECE, $CDTIPORECE) {
        try {
            switch ($IDTPTEF) {
                case '0':
                    $retornoContinua = $this->sitefPinpad->continuaSitef($valorTotal, $IDTIPORECE, $CDTIPORECE);
                    break;
                case '2':
                    $retornoContinua = $this->capptaPinpad->continuaCappta();
                    break;
                default:
                    $retornoContinua = array(
                        'error'               => true,
                        'finishedTransaction' => true,
                        'message'             => 'Modo TEF não encontrado. Configura a parametrização. (Coluna IDTPTEF).' 
                    );
                    break;
            }
            return $retornoContinua;
        } catch (\Exception $e) {
            return array(
                'error' => true,
                'message' => 'Mensagem: ' . $e->getMessage(),
                'finishedTransaction' => true
            );
        }
    }
    
    public function finalizaTransacao($IDTPTEF) {
        try {
            switch ($IDTPTEF) {
                case '0':
                    $retornoFinaliza = $this->sitefPinpad->finalizaTransacao();
                    break;
                case '2':
                    $retornoFinaliza = $this->capptaPinpad->finalizaTransacao();
                    break;
                default:
                    $retornoFinaliza = array(
                        'error'               => true,
                        'finishedTransaction' => true,
                        'message'             => 'Modo TEF não encontrado. Configura a parametrização. (Coluna IDTPTEF).' 
                    );
                    break;
            }
            return $retornoFinaliza;
        } catch (\Exception $e) {
            return array(
                'error' => false,
                'message' => 'Transação realizada com sucesso.',
                'finishedTransaction' => true
            );
        }
    }

    public function validaParametrosTef($CDFILIAL, $CDCAIXA) {
    	$retorno = array();

    	$params = array(
    		'CDFILIAL' => $CDFILIAL,
    		'CDCAIXA' => $CDCAIXA
    	);

    	$dadosTef = $this->entityManager->getConnection()->fetchAssoc(self::BUSCA_DADOS_TEF, $params);

    	if (empty($dadosTef['CDLOJATEF'])) {
    		$retorno = array(
    			'error' => true,
    			'message' => 'Loja TEF não parametrizado para o caixa.'
    		);
    	} elseif (empty($dadosTef['CDTERTEF'])) {
    		$retorno = array(
    			'error' => true,
    			'message' => 'Terminal TEF não parametrizado para o caixa.'
    		);
    	} elseif (empty($dadosTef['DSENDIPSITEF'])) {
    		$retorno = array(
    			'error' => true,
    			'message' => 'Endereço IP do servidor TEF não parametrizado para o caixa.'
    		);
    	} else {
    		$retorno = array(
    			'error' => false,
    			'tefConfigurationParams' => $dadosTef
    		);
    	}

    	return $retorno;
    }

    const BUSCA_DADOS_TEF = "
        SELECT CDLOJATEF, CDTERTEF, DSENDIPSITEF
          FROM CAIXA
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
    ";

}
