function RestrictProductController(LoginRepository, RestrictProductService, templateManager, ScreenService, RestrictValueService, ProductRestrictionsRepository, FamilyProductsRepository){

    /* Runs when the page is entered. Gets a list of families and any product restrictions they might have. */
    this.getProductRestrictions = function(gridWidget){
        LoginRepository.findOne().then(function (consumerData){
            ProductRestrictionsRepository.clearAll().then(function (){
                RestrictProductService.getProductRestrictions(consumerData['CDFILIAL'], consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR']).then(function (foodRestrictions){
                    gridWidget.activate();
                    gridWidget.reload();
                    templateManager.updateTemplate();
                    gridWidget.activate();
                });
            });
        });
    };

    this.showAddProductRestrictionPopup = function(widget){
        LoginRepository.findOne().then(function (consumerData){
            FamilyProductsRepository.clearAll().then(function (){
                RestrictProductService.getConsumerFamilies(consumerData['CDFILIAL'], consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR']).then(function (families){
                    widget.setCurrentRow({});

                    widget.getField('NMFAMILISALD').reload();

                    widget.getField('QTCONSDIARALFIELD').isVisible = true;
                    widget.currentRow.nullQuantityCheckBox = "N"
                    widget.currentRow.QTCONSDIARALFIELD = 1;

                    widget.getField('NMPRODUTO1').reload();

                    if (families.length == 1){
                        widget.currentRow.NMFAMILISALD = families[0].NMFAMILISALD;
                        widget.currentRow.CDFAMILISALD = families[0].CDFAMILISALD;
                        if (widget.getField('NMFAMILISALD').dataSource.data.length > 0){
                            widget.getField('NMFAMILISALD').setValue(widget.getField('NMFAMILISALD').dataSource.data[0].NMFAMILISALD);
                        }

                        this.fillFamilyProducts(families[0].CDFAMILISALD, widget);
                    }

                    templateManager.updateTemplate();
                    ScreenService.openPopup(widget);
                }.bind(this));
            }.bind(this));
        }.bind(this));
    };

    /* Fills the products according to the family selected when adding a new product restriction. */
    this.fillFamilyProducts = function(CDFAMILISALD, widget){
        LoginRepository.findOne().then(function (consumerData){
            RestrictProductService.getFamilyProducts(consumerData['CDFILIAL'], CDFAMILISALD, consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR']).then(function (products){
                widget.getField('NMPRODUTO1').reload();
                widget.currentRow['CDPRODUTO'] = undefined;
                widget.currentRow['NMPRODUTO'] = undefined;
                templateManager.updateTemplate();
            });
        });
    };

    /* Opens a popup that allows the update of a product restriction. */
    this.showUpdateProductRestrictionPopup = function(row, widget) {
        widget.setCurrentRow(row);
        widget.label = row.DSRESTRICAO;
        if (row.IDPAI) {
            if (row.QTCONSDIARAL == '0' || row.QTCONSDIARAL == 'não pode consumir'){
                widget.fields[1].value('Y');
                widget.fields[2].isVisible = false;
            }
            else {
                widget.fields[1].value('N');
                widget.fields[2].isVisible = true;
            }

            delete widget.dataSource;
            if (row.QTCONSDIARALFIELD == "" || row.QTCONSDIARALFIELD == "0"){
                row.QTCONSDIARALFIELD = 1;
            }

            ScreenService.openPopup(widget);
        }
    };

    /* Adds a product restriction. */
    this.addProductRestriction = function (row, widget){
        if (row.CDFAMILISALD){
            if (row.CDPRODUTO){
                if(row.cdWeekDays && row.cdWeekDays.length > 0){
                    LoginRepository.findOne().then(function (consumerData){
                        if (row.nullQuantityCheckBox === 'Y'){
                            row.QTCONSDIARAL = 0;
                            row.QTCONSDIARALFIELD = 0;
                        }
                        ProductRestrictionsRepository.clearAll().then(function (){
                            RestrictProductService.addRestrictionFood(consumerData['CDFILIAL'], consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR'], row.CDFAMILISALD, row.CDPRODUTO, row.QTCONSDIARALFIELD, row.cdWeekDays).then(function (restrictionsFood){
                                widget.dataSource.data = restrictionsFood;
                                templateManager.updateTemplate();
                                ScreenService.closePopup();
                                widget.activate();
                            });
                        });
                    });
                } else{
                    ScreenService.showMessage("Informe os dias da semana para o bloqueio.");
                }
            } else {
                ScreenService.showMessage("Informe o produto.");
            }
        } else {
            ScreenService.showMessage("Informe a loja.");
        }
    }

    /* Updates a product restriction. */
    this.updateProductRestriction = function (row, widget, fieldName){
        LoginRepository.findOne().then(function (consumerData){
            if (row.nullQuantityCheckBox === 'Y'){
                row.QTCONSDIARAL = 0;
                row.QTCONSDIARALFIELD = 0;
            }
            ProductRestrictionsRepository.clearAll().then(function (){
                RestrictProductService.updateRestrictionFood(row.QTCONSDIARALFIELD,consumerData['CDFILIAL'], consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR'], row.IDPAI, row.CDPRODUTO, row.cdWeekDays).then(function (restrictionsFood){
                    widget.dataSource.data = restrictionsFood;
                    templateManager.updateTemplate();
                    ScreenService.closePopup();
                    widget.activate();
                });
            });
        });
    };

    /* Deletes a product restriction. */
    this.deleteProductRestriction = function (fieldName, gridWidget, row){
        ScreenService.confirmMessage(
            'Deseja remover este produto da restrição?',
            'question',
            function(){
                ProductRestrictionsRepository.clearAll().then(function (){
                    LoginRepository.findOne().then(function (consumerData){
                        RestrictProductService.deleteRestrictionFood(consumerData['CDFILIAL'], consumerData['CDCLIENTE'], consumerData['CDCONSUMIDOR'], row.IDPAI, row.CDPRODUTO).then(function (foodRestrictions){
                            gridWidget.dataSource.data = foodRestrictions;
                            templateManager.updateTemplate();
                            ScreenService.closePopup();
                            gridWidget.activate();
                        });
                    });
                });
            },
            function(){}
        );
    };

    /* Hides and shows the field that controls product quantity. */
    this.enableQuantitySelection = function(check, quantitySelect){
        if (check == undefined || check == 'N'){
            quantitySelect.isVisible = true;
        }
        else {
            quantitySelect.isVisible = false;
        }
    };

};

Configuration(function(ContextRegister){
    ContextRegister.register('RestrictProductController', RestrictProductController);
});